/*
 *  C140DEF.H
 *
 *  Description:    This file defines the dirver dependent constants, macros
 *                  and structures used in C140 driver.
 *                  This file is used in the NDIS 4.0 miniport driver.
 *
 *  Copyright (c) 1997 by D-Link Corporation. All rights reserved.
 *
 *  Created by David Hsieh
 *
 *  Revision History:
 *
 *  Name        Date        Ver     Brief
 *  ----------- ----------- ------- -----------------------------------------
 *  David Hsieh 1997-July   Beta    Create NDIS 4.0 miniport driver.
 *
 */
#ifndef C140DEF_H
#define C140DEF_H


/* NDIS driver spec define */
#define DRV_NDIS_MAJOR_VERSION          4
#define DRV_NDIS_MINOR_VERSION          0

/* Error log values */
#define DRV_ERRMSG_REGISTRY             (tULONG)0x01
#define DRV_ERRMSG_ALLOC_MEMORY         (tULONG)0x02
#define DRV_ERRMSG_SROM                 (tULONG)0x03
#define DRV_ERRMSG_MEDIA                (tULONG)0x04
#define DRV_ERRMSG_LOAD_CAM             (tULONG)0x05
#define DRV_ERRMSG_INIT_DEVICE          (tULONG)0x06
#define DRV_ERRMSG_SYSTEN_ERROR         (tULONG)0x07
#define DRV_ERRMSG_TXM_JABBER_TIMEOUT   (tULONG)0x08

/* default pci setting */
#define DRV_DEFAULT_CFCS                0x5

/* Maximum buffer size (1536) */
#define DRV_MAX_CACHE_LINE_SIZE         128
#define DRV_MAX_BUFFER_SIZE             (12 * DRV_MAX_CACHE_LINE_SIZE)

/* Minimal size of a transmit packet to be directly mapped into the transmit */
/* ring. Under this threshold, the packet is copied into a preallocated      */
/* transmit buffer to avoid the (expensive) physical mapping translation.    */
#define DRV_MIN_TX_SIZE                 256

/* Number and size of allocated Tx buffers. */
/* The number MUST BE a power of 2. */
#define DRV_NUM_OF_MAX_TX_BUFFERS       4
#define DRV_MAX_TX_BUFFER_SIZE          DRV_MAX_BUFFER_SIZE

#define DRV_NUM_OF_MIN_TX_BUFFERS       16
#define DRV_MIN_TX_BUFFER_SIZE          DRV_MIN_TX_SIZE

/* Physical mapping registers allocated to the adapter */
#define DRV_MIN_MAP_REGISTERS           4
#define DRV_MAX_MAP_REGISTERS           64

/* Maximum number of physical segments Chip140 accepts in a single Ndis */
/* buffer. Above this threshold, the packet is copied into a single     */
/* physically contiguous buffer                                         */
#define DRV_MAX_SEGMENTS                8

/* Reserved number of descriptor for setup packet */
#define DRV_NUM_OF_SETUP_DESC           2

/* Number of segment per descriptor which we used for transmit. */
#define DRV_NUM_OF_SEGMENT_PER_DESC     1

/* The maximum multicast address supported by this driver. */
#define DRV_MAX_MULTICAST_ADDRESSES     36

/* Read/Write CSR macro */
#define mREADCSR(_Index,_Value)     NdisRawReadPortUlong(       \
                                        pAdapter->rCsr[_Index], \
                                        (PULONG)(_Value)        \
                                        );
#define mWRITECSR(_Index,_Value)    NdisRawWritePortUlong(      \
                                        pAdapter->rCsr[_Index], \
                                        (ULONG)(_Value)         \
                                        );

#define DRV_RDES_ERROR_MASK (tULONG)(RDES0_OF   \
                                   | RDES0_CE   \
                                   | RDES0_RW   \
                                   | RDES0_CS   \
                                   | RDES0_TL   \
                                   | RDES0_RF   \
                                   | RDES0_DE)

#define DRV_TDES_ERROR_MASK (tULONG)(TDES0_UF   \
                                   | TDES0_EC   \
                                   | TDES0_LC   \
                                   | TDES0_NC   \
                                   | TDES0_LO   \
                                   | TDES0_TO)


/* CSR12 definition for DFE-500C1 */
typedef union tCSR12_BITS
{
    struct 
    {
        ulong reserved_0                      : 3;
        ulong duplex_mode                     : 1;
        ulong line_speed                      : 1;
        ulong transmit                        : 1;
        ulong receive                         : 1;
        ulong link_status                     : 1;
        ulong general_purpose_ctrl            : 1;
        ulong reserved_1                      :23;
    } bits;
    ulong lw;
} tCSR12_BITS;



/* receive descriptor structure */
typedef struct tRX_DESC
{
//  tRX_DESC_HW;   //get rid of gcc error

    tUSHORT         wStatus;
//    tUSHORT         wFrameLength;   //marked by becker for tulip_rx()
    short           wFrameLength;
    tULONG          dwCommand;
    tULONG          dwAddress1;
    tULONG          dwAddress2;

    /* driver specified */
    struct  tRX_DESC *pNext;     /* Point to next receive descriptor. */
    struct  sk_buff *rx_skbuff;

} tRX_DESC, *tPRX_DESC;


/* Tranmit descriptor structure */
typedef struct tTX_DESC
{
//  tTX_DESC_HW;    //get rid of gcc's error

    tUSHORT         wStatus;
    tUSHORT         wOwnBit;
    tULONG          dwCommand;
    tULONG          dwAddress1;
    tULONG          dwAddress2;

    struct tTX_DESC *pNext;         /* Point to next transmit descriptor.   */
    /* definition for Linux driver */
    struct  sk_buff *tx_skbuff;

} tTX_DESC, *tPTX_DESC;


/* Reserved descriptors for setup packet */
#define NUM_OF_SETUP_DESC           2

/* Size of packet array */
#define DRV_PACKET_ARRAY_SIZE       32

/* MAC Adapter structure */
typedef struct tADAPTER
{
    /* Public variables */
    tPUB                        Pub;
    /* Control and Status register map */
    tPORT                       rCsr[C140_TOTAL_CSRS];

    tMEDIA_CTRL                 MediaCtrlBlk;
    /* Target board's CFID & Rev number*/
    tULONG                      dwCFID;
    tUCHAR                      cRevNo;

    /* 100Mbps transmit threshold control bit control */
    tUCHAR                      cCurTr100;
    tUCHAR                      cCurTr10;
    tBOOLEAN                    bLineSpeed100;
    tBOOLEAN                    bFDEnable;

    /* Descriptors Buffer Management */
    tUINT                       nDescriptorSize;
    tUINT                       nNumOfRxDescriptors;
    tUINT                       nNumOfTxDescriptors;

    tPRX_DESC                   pRxRing;
    tPTX_DESC                   pTxRing;

    /* Receive buffers */
    tUINT                       pFreeRxBuffer;

    /* Transmit/Receive descriptor management */
    tPTX_DESC                   pEnqueueTxDesc;
    tPTX_DESC                   pDequeueTxDesc;
    tPRX_DESC                   pDequeueRxDesc;

    /* Filter management */
    int                           SetupBuffer[48];
    /* transmit packet */
    int                       nNumUnusedTxDesc;

    /* Current network address we used */

    /* Linux's Statistics Variables see include file 'if_ether.h' for details*/
    struct enet_statistics stats;

    /* definitions for Linux */
    struct device       *next_module;
    tULONG              csr6;
    tULONG              csr7;
    unsigned int        tx_full:1;
    unsigned int        tx_full_duplex:1;
    struct timer_list   timer;

    /* default media variable */
    char                default_media;

   /* to keep the previous CSR12 value for DFE-500TX */
    tCSR12_BITS         OldCSR12Value;
   /* to keep the previous connection type for DC21143 */
    tBYTE               dc21143_pre_connection_type;

    int                 pad0,pad1;          /* used for 8-byte alignment */
} tADAPTER, *tPADAPTER;

#endif /* c140def.h */
