/*
 *  C140KWD.H
 *
 *  Description:    This file defines all the custom keywords of the
 *                  driver.
 *
 *  Copyright (c) 1997 by D-Link Corporation. All rights reserved.
 *
 *  Revision History:
 *
 *  Name        Date        Ver     Brief
 *  ----------- ----------- ------- ------------------------------------------
 *
 */

#ifndef C140KWD_H
#define C140KWD_H

/* Connection type option string */
#define KWD_NWAYAUTO_TXT                "AUTOSENSE"
#define KWD_10BASET_TXT                 "10BASET"
#define KWD_10BASETFD_TXT               "10BASETFD"
#define KWD_100BASETX_TXT               "100BASETX"
#define KWD_100BASETXFD_TXT             "100BASETXFD"
#define KWD_NWAY10BASET_TXT             "NWAY10BASET"
#define KWD_NWAY10BASETFD_TXT           "NWAY10BASETFD"
#define KWD_NWAY100BASETX_TXT           "NWAY100BASETX"
#define KWD_NWAY100BASETXFD_TXT         "NWAY100BASETXFD"

/* Other option string */
#define KWD_PBL_TXT                     "PBL=%X"
#define KWD_CAL_TXT                     "CAL=%X"
#define KWD_TR10_TXT                    "TR10=%X"
#define KWD_TR100_TXT                   "TR100=%X"
#define KWD_RXBUF_TXT                   "RXBUF=%X"
#define KWD_TXBUF_TXT                   "TXBUF=%X"
#define KWD_STOREFORWARD_TXT            "STOREFORWARD"
#define KWD_CFLT_TXT                    "LATENCYTIMER=%X"

/*
    Connection type defines:
    DO NOT change the setting of this field. Any other additional connection
    type should be appended, do not modify the existing connection type setting
*/
#define CT_NWAY_AUTO                    0       /* Nway Autosense */
#define CT_10BASE_T                     1       /* 10Base-T */
#define CT_10BASE_T_FD                  2       /* 10Base-T full duplex */
#define CT_100BASE_TX                   3       /* 100Base-Tx */
#define CT_100BASE_TX_FD                4       /* 100Base-Tx full duplex */
#define CT_NWAY_10BASE_T                5       /* Nway advertise 10Base-T only */
#define CT_NWAY_10BASE_T_FD             6       /* Nway advertise 10Base-T full duplex only */
#define CT_NWAY_100BASE_TX              7       /* Nway advertise 100Base-Tx only */
#define CT_NWAY_100BASE_TX_FD           8       /* Nway advertise 100Base-Tx full duplex onlu */

#define CT_DEFAULT                      CT_NWAY_AUTO


/* Programable Burst length */
/* Burst Length: indicates the maximum number of longwords to be transfer in
                 one DMA transaction. */
#define PBL_0_LONG_WORD                 0x00
#define PBL_1_LONG_WORD                 0x01
#define PBL_2_LONG_WORD                 0x02
#define PBL_4_LONG_WORD                 0x04
#define PBL_8_LONG_WORD                 0x08
#define PBL_16_LONG_WORD                0x10
#define PBL_32_LONG_WORD                0x20
#define PBL_BITS_MASK                   0x3F
#define PBL_BITS_SHIFT                  8

#define PBL_DEFAULT                     PBL_32_LONG_WORD
#define PBL_MAX                         PBL_16_LONG_WORD
#define PBL_MIN                         PBL_0_LONG_WORD

/* Cache alignment : Programable address boundaries for data burst stop. */
#define CAL_RESERVED                    0x0     /* Reserved */
#define CAL_8_LONG_WORD                 0x1     /* 8-longword boundary alignment */
#define CAL_16_LONG_WORD                0x2     /* 16-longword boundary alignment */
#define CAL_32_LONG_WORD                0x3     /* 32-longword boundary alignment */
#define CAL_BITS_MASK                   0x3
#define CAL_BITS_SHIFT                  14

#define CAL_DEFAULT                     CAL_32_LONG_WORD
#define CAL_MAX                         CAL_32_LONG_WORD
#define CAL_MIN                         CAL_8_LONG_WORD

/* Threshold control bits : Controls the selected threshold level for the
                            transmit FIFO. */
#define TR10_72                         0x0     /* 72 bytes in 10Mbps */
#define TR10_96                         0x1     /* 96 bytes in 10Mbps */
#define TR10_128                        0x2     /* 128 bytes in 10Mbps */
#define TR10_160                        0x3     /* 160 bytes in 10Mbps */
#define TR100_128                       0x0     /* 128 bytes in 100Mbps */
#define TR100_256                       0x1     /* 256 bytes in 100Mbps */
#define TR100_512                       0x2     /* 512 bytes in 100Mbps */
#define TR100_1024                      0x3     /* 1024 bytes  in 100Mbps */
#define TR_BITS_MASK                    0x3
#define TR_BITS_SHIFT                   14

#define TR10_DEFAULT                    TR10_72
#define TR10_MAX                        TR10_160
#define TR10_MIN                        TR10_72
#define TR100_DEFAULT                   TR100_128
#define TR100_MAX                       TR100_1024
#define TR100_MIN                       TR100_128

/* Rx buffer number */
#define RXBUF_DEFAULT                   4
#define RXBUF_MAX                       4
#define RXBUF_MIN                       4

/* Tx buffer number */
#define TXBUF_DEFAULT                   4
#define TXBUF_MAX                       4
#define TXBUF_MIN                       4

/* Latency timer */
#define CFLT_MIN                        0x0
#define CFLT_MAX                        0xFF
#define CFLT_DEFAULT                    CFLT_MAX
#define CFLT_MIN_WE_ACCEPT              0x50
#define CFLT_BITS_MASK                  0xFF
#define CFLT_BITS_SHIFT                 8


/* User selection storage structure */
typedef struct tUSER_SEL
{
    /* Connection type */
    tUSHORT             wConnectionType;
    tBOOLEAN            bCTPresent;

    /* PCI Latency timer */
    tUCHAR              cLatencyTimer;
    tBOOLEAN            bCFLTPresent;

    /* Programable burst length */
    tUCHAR              cBurstLength;
    tBOOLEAN            bPBLPresent;

    /* Cache alignment */
    tUCHAR              cCacheAlignment;
    tBOOLEAN            bCALPresent;

    /* 10Mbps threshold control bits */
    tUCHAR              cTxThreshold10;
    tBOOLEAN            bTR10Present;

    /* 100Mbps thresholld control bits */
    tUCHAR              cTxThreshold100;
    tBOOLEAN            bTR100Present;

    /* Rx buffer number */
    tUCHAR              cRxBufferNumber;
    tBOOLEAN            bRXBUFPresent;

    /* Tx buffer number */
    tUCHAR              cTxBufferNumber;
    tBOOLEAN            bTXBUFPresent;

    /* Store and forward */
    tBOOLEAN            bStoreAndForward;

#if 0   /* No need in C ODI driver */
    /* Network Address */
    tUCHAR              cNetworkAddress[6];
    tBOOLEAN            bNAPresent;

    /* Slot number */
    tUCHAR              cSlot;
    tBOOLEAN            bSlotPresent;
#endif

} tUSER_SEL, *tPUSER_SEL;


#endif
