/*
 *  C140REG.H
 *
 *  Description:    This file defines the constants of Chip140 CSRs.
 *
 *  Copyright (c) 1997 by D-Link Corporation. All right reserved.
 *
 *  Created by David Hsieh
 *
 *  Revision History:
 *
 *  Name        Date        Ver     Brief
 *  ----------- ----------- ------- ------------------------------------------
 *  David Hsieh             0.90    Creating
 *
 */
#ifndef C140REG_H
#define C140REG_H



/********************************************************************
 *
 *  Control and Status registers define
 *
 ********************************************************************/

/*  CSR0 : Bus mode register */
#define CSR0_RESET              0x00000001
#define CSR0_BAR                0x00000002
#define CSR0_DSL_MASK           0x0000007C
#define CSR0_BLE                0x00000080
#define CSR0_PBL                0x000003F0
#define CSR0_CAL                0x0000C000
#define CSR0_TAP                0x000E0000
#define CSR0_DBO                0x00100000
#define CSR0_RML                0x00200000


/*  CSR5 : Status register */
#define CSR5_TI                 0x00000001
#define CSR5_TPS                0x00000002
#define CSR5_TU                 0x00000004
#define CSR5_TJT                0x00000008
#define CSR5_UNF                0x00000020
#define CSR5_RI                 0x00000040
#define CSR5_RU                 0x00000080
#define CSR5_RPS                0x00000100
#define CSR5_RWT                0x00000200
#define CSR5_ETI                0x00000400
#define CSR5_GTE                0x00000800
#define CSR5_FBE                0x00002000
#define CSR5_AIS                0x00008000
#define CSR5_NIS                0x00010000
#define CSR5_RS                 0x000E0000
#define CSR5_TS                 0x00700000
#define CSR5_EB                 0x03100000

/* Transmit process state */
#define TPS_STOPED              ((tULONG)(0x0 << 23))
#define TPS_FETCHING_DESC       ((tULONG)(0x1 << 23))
#define TPS_WAIT_FOR_END_OF_TX  ((tULONG)(0x2 << 23))
#define TPS_READ_BUFFER         ((tULONG)(0x3 << 23))
#define TPS_RESERVED            ((tULONG)(0x4 << 23))
#define TPS_RUN_SETUP_PACKET    ((tULONG)(0x5 << 23))
#define TPS_SUSPENDED           ((tULONG)(0x6 << 23))
#define TPS_CLOSE_TX_DESC       ((tULONG)(0x7 << 23))

/* Receive process state */
#define RPS_STOPED              ((tULONG)(0x0 << 20))
#define RPS_FETCHING_DESC       ((tULONG)(0x1 << 20))
#define RPS_CHECK_END_OF_RX     ((tULONG)(0x2 << 20))
#define RPS_WAIT_FOR_RX         ((tULONG)(0x3 << 20))
#define RPS_SUSPENDED           ((tULONG)(0x4 << 20))
#define RPS_CLOSE_RX_DESC       ((tULONG)(0x5 << 20))
#define RPS_FLUSH_FIFO          ((tULONG)(0x6 << 20))
#define RPS_FILL_RX_BUFFER      ((tULONG)(0x7 << 20))

/*  CSR6 : Operation mode register */
#define CSR6_HP                 0x00000001
#define CSR6_SR                 0x00000002
#define CSR6_HO                 0x00000004
#define CSR6_PB                 0x00000008
#define CSR6_IF                 0x00000010
#define CSR6_SB                 0x00000020
#define CSR6_PR                 0x00000040
#define CSR6_PM                 0x00000080
#define CSR6_FD                 0x00000200
#define CSR6_OM                 0x00000C00
#define CSR6_FC                 0x00001000
#define CSR6_ST                 0x00002000
#define CSR6_TR                 0x0000C000
#define CSR6_CA                 0x00020000
#define CSR6_PS                 0x00040000
#define CSR6_HBD                0x00080000
#define CSR6_IMM                0x00100000
#define CSR6_SF                 0x00200000
#define CSR6_TTM                0x00400000
#define CSR6_PCS                0x00800000
#define CSR6_SCR                0x01000000

#define CSR6_MUST_BE_ZERO       0x08000100
#define CSR6_MUST_BE_1          0x02000000
#define CSR6_BITS_CHECK         (CSR6_SCR | CSR6_PCS | CSR6_TTM |          \
                                 CSR6_SF  | CSR6_IMM | CSR6_TR )


/*  CSR7 : Interrupt enable register */
#define CSR7_TI                 0x00000001
#define CSR7_TS                 0x00000002
#define CSR7_TU                 0x00000004
#define CSR7_TJ                 0x00000008
#define CSR7_UN                 0x00000020
#define CSR7_RI                 0x00000040
#define CSR7_RU                 0x00000080
#define CSR7_RS                 0x00000100
#define CSR7_RW                 0x00000200
#define CSR7_ETE                0x00000400                                                      #define
#define CSR7_GPT                0x00000800
#define CSR7_FBE                0x00002000
#define CSR7_AI                 0x00008000
#define CSR7_NI                 0x00010000


/*  CSR9 : Boot ROM, Serial ROM and MII Management register */
#define CSR9_DATA               0x000000FF
#define CSR9_REG                0x00000400
#define CSR9_SR                 0x00000800
#define CSR9_BR                 0x00001000
#define CSR9_WR                 0x00002000
#define CSR9_RD                 0x00004000
#define CSR9_MDC                0x00010000
#define CSR9_MDO                0x00020000
#define CSR9_MII                0x00040000
#define CSR9_MDI                0x00080000


/*  CSR11 : General-Purpose timer register */
#define CSR11_COM               0x00010000


/*  CSR12 : Feneral-Purpose Port Register */
#define CSR12_GPC               0x00000100



#endif
