/*
 *  CHIP140.H
 *
 *  Description:    This file defines the driver independent constants, macros
 *                  and structures used in C140 driver.
 *
 *  Copyright (c) 1997 by D-Link Corporation. All rights reserved.
 *
 *  Created by David Hsieh
 *
 *  Revision History:
 *
 *  Name        Date        Ver     Brief
 *  ----------- ----------- ------- ------------------------------------------
 *  David Hsieh 1997-July   Beta    Create NDIS 4.0 miniport drivers.
 *
 *
 */
#ifndef CHIP140_H
#define CHIP140_H



/* Control and Status register defines */
#define C140_TOTAL_CSRS             17
#define C140_CSR_OFFSET             8
#define C140_IO_RANGE               (C140_CSR_OFFSET * C140_TOTAL_CSRS)

/* Chip 140 CFID define */
#define CHIP140_CFID                0x00091011
#define MX98713_CFID                0x051210d9

/* Serial EEPROM valid data length (in bytes) */
#define MAX_SROM_SIZE               128

/* Useful define & macros */
#define mIS_MULTICAST(_addr)        ((*(tPUCHAR)(_addr) & 1) == 1)
#define mIS_BROADCAST(_addr)        ((*(tPULONG)(_addr) == 0xFFFFFFFF) &&    \
                                     (*(tPUSHORT)((PUCHAR)(_addr) + 4) ==    \
                                                                0xFFFF))
#define mIS_NULL_ADDRESS(_addr)     ((*(tPULONG)(_addr) == 0) &&             \
                                     (*(tPUSHORT)((tPUCHAR)(_addr) + 4) == 0))

#define mMAX(a,b)                   ((a)>(b) ? (a) : (b))
#define mMIN(a,b)                   ((a)<(b) ? (a) : (b))


/***********************************
 *
 *  TX and RX strucutre definition
 *
 ***********************************/



/* Descriptor owner bits */
#define DESC_OWN_BIT                0x8000  /* Owner bit */
#define DESC_OWN_BY_NIC             0x8000  /* When set, owned by Chip140 */
#define DESC_OWN_BY_HOST            0x0000  /* When reset, owned by host */

#define NUM_OF_SEGMENT_PER_DESC     2

/* RX descriptor, status longword bits assignment */
#define RDES0_OF                    0x0001  /* Overflow */
#define RDES0_CE                    0x0002  /* CRC error */
#define RDES0_DB                    0x0004  /* Dribbling bit */
#define RDES0_RE                    0x0008  /* Report on MII error */
#define RDES0_RW                    0x0010  /* Receive watchdog */
#define RDES0_FT                    0x0020  /* Frame type */
#define RDES0_CS                    0x0040  /* Collision seen */
#define RDES0_TL                    0x0080  /* Frame too long */
#define RDES0_LS                    0x0100  /* Last descriptor */
#define RDES0_FS                    0x0200  /* First descriptor */
#define RDES0_MF                    0x0400  /* Multicast frame */
#define RDES0_RF                    0x0800  /* Runt frame */
#define RDES0_DT                    0x3000  /* Data type */
#define RDES0_DE                    0x4000  /* Descriptor error */
#define RDES0_ES                    0x8000  /* Error summary */

#define RDES0_FL_BITS_MASK          0x3FFF  /* Frame length */

#define RDES1_RCH                   0x01000000  /* Second address chained */
#define RDES1_RER                   0x02000000  /* Receive End of Ring */



/* TX descriptor, status longword bits assignment */
#define TDES0_DE                    0x0001  /* Deffered */
#define TDES0_UF                    0x0002  /* Underflow */
#define TDES0_LF                    0x0004  /* Link fail report */
#define TDES0_CC                    0x0078  /* Collision count */
#define TDES0_HF                    0x0080  /* Heart beat fail */
#define TDES0_EC                    0x0100  /* Excessive Collisions */
#define TDES0_LC                    0x0200  /* Late collision */
#define TDES0_NC                    0x0400  /* No carrier */
#define TDES0_LO                    0x0800  /* Loss of carrier */
#define TDES0_TO                    0x4000  /* Transmit jabber timeout */
#define TDES0_ES                    0x8000  /* Error summary */

#define TDES0_CC_BITS_SHIFT         3

#define TDES1_IC                    0x80000000  /* Interrupt on completion */
#define TDES1_LS                    0x40000000  /* Last segment */
#define TDES1_FS                    0x20000000  /* First segment */
#define TDES1_FT1                   0x10000000  /* Filtering type */
#define TDES1_SET                   0x08000000  /* Setup packet */
#define TDES1_AC                    0x04000000  /* Add CRC disable */
#define TDES1_TER                   0x02000000  /* Transmit end of ring */
#define TDES1_TCH                   0x01000000  /* Second address chained */
#define TDES1_DPD                   0x00800000  /* Diable padding */
#define TDES1_FT0                   0x00400000  /* Filtering type */
#define TDES1_HASH_FILTERING        TDES1_FT0

#define TDES1_2ND_BUFFER_BITS_SHIFT 11

/* Use by setup packet */
#define SETUP_PACKET_TIMEOUT        50  /* *10ms */
#define SETUP_BUFFER_SIZE           192
#define SETUP_PERFECT_ENTRIES       16
#define SETUP_HASH_ENTRIES          512
#define MAX_MULTICAST_PERFECT       14

/* Filtering mode define */
typedef enum tFILTERING_MODE
{
    ePERFECT_FILTERING,
    eHASH_FILTERING
} tFILTERING_MODE, *tPFILTERING_MODE;

/* Setup packet buffer */
typedef union tSETUP_BUFFER
{
#if 0
    struct
    {
        tULONG NetworkAddress[SETUP_PERFECT_ENTRIES][3];
    } Perfect;
    struct
    {
        tULONG Filter[32];
        tULONG Mbz1[7];
        tULONG NetworkAddress[3];
        tULONG Mbz2[6];
    } Hash;
#endif
tULONG eaddr[48];
} tSETUP_BUFFER, *tPSETUP_BUFFER;

/* Reset state define */
typedef enum tRESET_STATE
{
    eRESET_IDLE,
    eRESET_IN_PROGRESS,
    eRESET_UPDATE_MCST,
    eUPDATE_MCST_TIMEOUT
} tRESET_STATE;

/* Line speed define */
typedef enum tLINE_SPEED
{
    eLINE_SPEED_10M=10,
    eLINE_SPEED_100M=100
} tLINE_SPEED;

/* Receive descriptor */
typedef struct tRX_DESC_HW
{
    tUSHORT         wStatus;
//    tUSHORT         wFrameLength;   //marked by becker for tulip_rx()
    short           wFrameLength;
    tULONG          dwCommand;
    tULONG          dwAddress1;
    tULONG          dwAddress2;
} tRX_DESC_HW;

/* Transmit descriptor */
typedef struct tTX_DESC_HW
{
    tUSHORT         wStatus;
    tUSHORT         wOwnBit;
    tULONG          dwCommand;
    tULONG          dwAddress1;
    tULONG          dwAddress2;
} tTX_DESC_HW;

#endif
