/******************************************************************************
 *
 * File:    DLKPCI.C
 *
 * Description:
 *      Implementation of D-Link DFE-500TX adapter card for Linux.
 *
 * Driver's Stage:
 *      v0.20 ....Support C1,C3 only    -- 2,July,1997 --Becker
 *      v0.20 ....Support D1            -- 12,Sep,1997 --Becker
 *      v0.21                           -- 24,Sep,1997 --Becker
 *            Added 'options' to support media because the D1 of the USA version
 *            fial to autosense succeefully with INTEL 100M HUB.
 *            optiopns supported values :    
 *                       0 -- Autosense
 *                       1 -- 10BaseT
 *                       2 -- 10BaseT Full Duplex
 *                       3 -- 100BaseTx 
 *                       4 -- 100BaseTx Full Duplex
 *
 *            usage :
 *                    insmod ethpci.o options=1
 * 
 ******************************************************************************/

//#define DEBUG_INTERRUPT
//#define DEBUG_LINK_CHANGE_INTR
//#define DEBUG_RX_INTR
//#define DEBUG_TX_DESC
//#define DEBUG_RX_DESC
//#define DEBUG_CLOSE
//#define DEBUG_CSR5
//#define DEBUG_CSR7
//#define DEBUG_CSR6
//#define DEBUG_CSR12
//#define DEBUG_MULTICAST

static const char *version = 
"PCI Fast Ethernet driver v0.21 9/18/97 dandy@eeipc3.ee.ccu.edu.tw\n";

#define SKBUFF_RX_COPYBREAK 200
#define TX_TIMEOUT  ((2000*HZ)/1000)
#define PKT_BUF_SZ  1536          /* Size of each temporary Rx buffer */

/* Keep the ring sizes a power of two for efficiency.
   Making the Tx ring too large decreases the effectiveness of channel
   bonding and packet priority.
   There are no ill effects from too-large receive rings. */
#define TX_RING_SIZE    16
#define RX_RING_SIZE    16

#define TULIP_TOTAL_SIZE        0x90

/* definintion for MXIC--Added by Becker Horng */
#ifndef PCI_VENDOR_ID_MXIC
#define PCI_VENDOR_ID_MXIC              0x10d9
#define PCI_DEVICE_ID_MXIC_MX98713      0x0512
#define PCI_DEVICE_ID_DEC_TULIP_21143   0x0019
#endif

#include <linux/config.h>
#ifdef MODULE
#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif
#include <linux/module.h>
#include <linux/version.h>
#else
#define MOD_INC_USE_COUNT
#define MOD_DEC_USE_COUNT
#endif

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/timer.h>
#include <linux/ptrace.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/malloc.h>
#include <linux/interrupt.h>
#include <linux/pci.h>
#include <linux/bios32.h>
#include <asm/processor.h>              /* Processor type for cache alignment. */
#include <asm/bitops.h>
#include <asm/io.h>
#include <asm/dma.h>

#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>

#include <linux/version.h>


/*
 * include files of our driver library
 */
#include "typesdef.h"
#include "chip140.h"
#include "pub.h"
#include "mii.h" /* for tPMII_PHY_INFO */
#include "mactophy.h"
#include "c140def.h"
#include "c140kwd.h"
#include "c140reg.h"
#include "debug.h"

#ifdef TULIP_DEBUG
int tulip_debug=TULIP_DEBUG;
#else
int tulip_debug=1;
#endif

#if defined (LINUX_VERSION_CODE) && LINUX_VERSION_CODE < 0x10300
#define RUN_AT(x) (x)                   /* What to put in timer->expires.  */
#define DEV_ALLOC_SKB(len) alloc_skb(len, GFP_ATOMIC)
#define virt_to_bus(addr)  ((unsigned long)addr)
#define bus_to_virt(addr) ((void*)addr)

#else  /* 1.3.0 and later */
#define RUN_AT(x) (jiffies + (x))
#define DEV_ALLOC_SKB(len) dev_alloc_skb(len + 2)
#endif

#if defined (LINUX_VERSION_CODE) && LINUX_VERSION_CODE < 0x10338
#ifdef MODULE
#if !defined(CONFIG_MODVERSIONS) && !defined(__NO_VERSION__)
char kernel_version[] = UTS_RELEASE;
#endif
#else
#undef MOD_INC_USE_COUNT
#define MOD_INC_USE_COUNT
#undef MOD_DEC_USE_COUNT
#define MOD_DEC_USE_COUNT
#endif
#endif /* 1.3.38 */

#if (LINUX_VERSION_CODE >= 0x10344)
#define NEW_MULTICAST
#include <linux/delay.h>
#endif

#ifdef SA_SHIRQ
#define FREE_IRQ(irqnum, dev) free_irq(irqnum, dev)
#define REQUEST_IRQ(i,h,f,n, instance) request_irq(i,h,f,n, instance)
#define IRQ(irq, dev_id, pt_regs) (irq, dev_id, pt_regs)
#else
#define FREE_IRQ(irqnum, dev) free_irq(irqnum)
#define REQUEST_IRQ(i,h,f,n, instance) request_irq(i,h,f,n)
#define IRQ(irq, dev_id, pt_regs) (irq, pt_regs)
#endif

/* This my implementation of shared IRQs, now only used for 1.2.13. */
#ifdef HAVE_SHARED_IRQ
#define USE_SHARED_IRQ
#include <linux/shared_irq.h>
#endif

/* The rest of these values should never change. */

static void tulip_timer(unsigned long data);

/* A table describing the chip types. */
static struct tulip_chip_table {
  int device_id;
  char *chip_name;
  int flags;
  void (*media_timer)(unsigned long data);
} tulip_tbl[] = {
  { PCI_DEVICE_ID_DEC_TULIP_FAST,  "DEC DC21140", 0, tulip_timer }, /* 21140 */
  { PCI_DEVICE_ID_DEC_TULIP_21143, "DEC DC21143", 0, tulip_timer }, /* 21143 */
  { PCI_DEVICE_ID_MXIC_MX98713,    "MXIC MX98713",0, tulip_timer }, /* MXIC's 100M controller */
  {0, 0, 0, 0},
};
/* This matches the table above. */
enum chips { DC21140=0,DC21143=1,MX98713=2};

/* Offsets to the Command and Status Registers, "CSRs".  All accesses
   must be longword instructions and quadword aligned. */
enum tulip_offsets {
        CSR0=0,    CSR1=0x08, CSR2=0x10, CSR3=0x18, CSR4=0x20, CSR5=0x28,
        CSR6=0x30, CSR7=0x38, CSR8=0x40, CSR9=0x48, CSR10=0x50, CSR11=0x58,
        CSR12=0x60, CSR13=0x68, CSR14=0x70, CSR15=0x78,CSR16=0x80,CSR17=0x88,
        CSR18=0x90, CSR19=0x98, CSR20=0x100 };

#ifdef MODULE
/* A list of all installed Tulip devices, for removing the driver module. */
static struct device *root_dlkpci_dev = NULL;
#endif

#define  AUTOSENSE        0
#define  F10BASET         1
#define  F10BASET_FD      2
#define  F100BASETX       3
#define  F100BASETX_FD    4
#define  MAX_MEDIA_TABLE  5

static int options[8]={0,0,0,0,0,0,0,0};
static struct force_media_table {
    char   *media_type_mesg;
    tBYTE   cConnection;
} force_media_tbl[MAX_MEDIA_TABLE+1] = {

    {/* 0 */           "AutoSense",                  0xff},
    {/* 1 */           "10BaseT",                    MII_10BASE_T},
    {/* 2 */           "10BaseT Full Duplex",        MII_10BASE_T_FD},
    {/* 3 */           "100BaseTx",                  MII_100BASE_TX},
    {/* 4 */           "100BaseTx Full Duplex",      MII_100BASE_TX_FD},
    {0,0},
};

/* functions' prototype */
static struct
device *tulip_probe1(struct device *dev, int ioaddr, int irq, int chip_id, int options);
static void set_force_media(struct device *dev);
static void select_media (struct device *dev);
static int tulip_open(struct device *dev);
static void tulip_timer(unsigned long data);
static void tulip_tx_timeout(struct device *dev);
static void tulip_init_ring(struct device *dev);
static int tulip_start_xmit(struct sk_buff *skb, struct device *dev);
static int tulip_rx(struct device *dev);
static void tulip_interrupt IRQ(int irq, void *dev_instance, struct pt_regs *regs);
static int tulip_close(struct device *dev);
static struct enet_statistics *tulip_get_stats(struct device *dev);
#ifdef NEW_MULTICAST
static void set_multicast_list(struct device *dev);
#else
static void set_multicast_list(struct device *dev, int num_addrs, void *addrs);
#endif

/* added by becker */
tRESULT bReadSROMWords(tPPUB pPub, tPORT Port, tPUSHORT pBuffer, tUINT nLen); /* in srom.c */
static void check_dfe500c1_connection(tPADAPTER tpadapter,int ioaddr);
static void showmedia_change_csr6(tPADAPTER tpadapter);
static tBYTE csr12_to_connection_type(tCSR12_BITS CSR12Value);
static void detect_21143_media(struct device *dev);


/* definitions for DC21143's auto negotiation */
#define LINK_CHANGE_INTERRUPT_MAX     100

/* defintions for variable "anc_ok" */
#define AUTOSENSE_FAIL                0
#define AUTOSENSE_SUCCESS             1
#define AUTOSENSE_AGAIN               2
#define AUTOSENSE_DYNAMIC             4 /* start to dynamic autosense */

unsigned char anc_ok; /* flag for ANC and dynamically autosense for DC21143 
                         In tulip_timer(), we issue a media sense when anc_ok=0*/
static int  link_change_intr;

/*-----------------------------------------------------------------------------
 * tulip_probe(): This function is used to probe a D-Link DFE500TX adapter card.
 * It should be used when booting.
 *
 *-----------------------------------------------------------------------------
 */
int tulip_probe(struct device *dev)
{
    int cards_found = 0;
    static int pci_index = 0;       /* Static, for multiple probe calls. */
    unsigned int pci_ioaddr;
    unsigned char pci_bus, pci_device_fn;

    if (pcibios_present()) {
        for (;pci_index < 0xff; pci_index++) {
            unsigned char pci_irq_line, pci_latency;
            unsigned short pci_command, vendor, device;
            unsigned int chip_idx = 0;
            if (pcibios_find_class (PCI_CLASS_NETWORK_ETHERNET << 8, pci_index,
                &pci_bus, &pci_device_fn) != PCIBIOS_SUCCESSFUL)
                break;

            pcibios_read_config_word(pci_bus, pci_device_fn, PCI_VENDOR_ID, &vendor);
            pcibios_read_config_word(pci_bus, pci_device_fn, PCI_DEVICE_ID, &device);
            pcibios_read_config_byte(pci_bus, pci_device_fn, PCI_INTERRUPT_LINE, &pci_irq_line);
            pcibios_read_config_dword(pci_bus, pci_device_fn, PCI_BASE_ADDRESS_0, &pci_ioaddr);

            /* must be dword alignment */
            pci_ioaddr &= ~3;
            if (tulip_debug>4)
                printk("PCI Vendor ID=[%04X] PCI Device ID=[%04X]\n",vendor,device);

            /* Vendor ID is defined in /usr/src/linux/include/linux/pci.h */
            if (vendor != PCI_VENDOR_ID_DEC&&vendor!=PCI_VENDOR_ID_MXIC)
                continue;

            for (chip_idx = 0; tulip_tbl[chip_idx].chip_name; chip_idx++)
                 if (device == tulip_tbl[chip_idx].device_id)
                     break;
            if (tulip_tbl[chip_idx].chip_name == 0){
                printk("Unknow PCI ethernet adapter %4.4x detected :not configured\n",device);
                continue;
            }
 
            if (check_region(pci_ioaddr, TULIP_TOTAL_SIZE))
                continue;
#ifdef MODULE
            dev = tulip_probe1(dev, pci_ioaddr, pci_irq_line, chip_idx, options[cards_found]);
#else
            dev = tulip_probe1(dev, pci_ioaddr, pci_irq_line, chip_idx, dev ? dev->mem_start : 0);
#endif
            if (dev) {
                /* Get and check the bus-master and latency values. */
               pcibios_read_config_word(pci_bus, pci_device_fn, PCI_COMMAND, &pci_command);
                if ( ! (pci_command & PCI_COMMAND_MASTER)) {
                     pci_command |= PCI_COMMAND_MASTER;
                     pcibios_write_config_word(pci_bus, pci_device_fn, PCI_COMMAND, pci_command);
                }/* if */
                pcibios_read_config_byte(pci_bus, pci_device_fn, PCI_LATENCY_TIMER, &pci_latency);
                if (pci_latency < 10) {
                    printk("PCI latency timer (CFLT) is unreasonably low at %d."
                            "  Setting to 255 clocks.\n", pci_latency);
                    pcibios_write_config_byte(pci_bus, pci_device_fn, PCI_LATENCY_TIMER, 255);
                } else
                   if (tulip_debug > 1)
                        printk("PCI latency timer (CFLT) is %#x.\n", pci_latency);

                /* Bring DC21143 out power-down mode */
               if (device==PCI_DEVICE_ID_DEC_TULIP_21143)
                   pcibios_write_config_dword(pci_bus, pci_device_fn, 0x40,0x40000000);
                dev = 0;
                cards_found++;
            }/* if */
        } /* for */
    } /* if */
    else 
       printk("Warning: PCI BIOS not found ! \n");

#if defined (MODULE)
    return cards_found;
#else
    return 0;
#endif
}


static struct device *tulip_probe1(struct device *dev, int ioaddr, int irq, int chip_id, int options)
{

    tADAPTER  *tpadapter = (tADAPTER *)dev->priv;
    tPUB      tPub;
    /* See note below on the multiport cards. */
    static unsigned char last_phys_addr[6] = {0x00, 'L', 'i', 'n', 'u', 'x'};
    static int last_irq = 0;
    int i;
    unsigned short chksum=0;
    unsigned char  ee_data[128];
    static char did_version=0;
   
    if (did_version++ ==0)
        printk(version);

    dev = init_etherdev(dev, sizeof(tPADAPTER));
    printk("%s: %s Fast Ethernet at %#3x,", dev->name, tulip_tbl[chip_id].chip_name, ioaddr);

    tPub.Port = ioaddr;
    /* Stop the chip's Tx and Rx processes. */
    outl(inl(ioaddr + CSR6) & ~0x2002, ioaddr + CSR6);
    /* Clear the missed-packet counter. */
    (volatile)inl(ioaddr + CSR8);
    //
    // Read SROM to get ehternet address
    //
    bReadSROMWords(&tPub,ioaddr + CSR9, (tPUSHORT)ee_data,sizeof(ee_data));

    for (i=0;i <6;i++) {
        dev->dev_addr[i]=ee_data[i+20];
        chksum += ee_data[i+20];
    } /* for */

    if (chksum ==0||chksum == 6*0xff){
        printk("EEPROM not found,");
        for (i=0;i<5;i++)
            dev->dev_addr[i] = last_phys_addr[i];
            dev->dev_addr[i] = last_phys_addr[i]+1;
        irq = last_irq;
    } /* if */

    //printk("ethernet address\n");
    for (i=0;i<6;i++)
        last_phys_addr[i] = dev->dev_addr[i];
    printk("Node ID=%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x,",last_phys_addr[0],
           last_phys_addr[1],last_phys_addr[2],last_phys_addr[3],last_phys_addr[4],
           last_phys_addr[5]);
    //print IRQ
    printk("IRQ %d\n", irq);
    last_irq = irq;

    // request region()
    /* We do a request_region() only to register /proc/ioports info. */
    request_region(ioaddr, TULIP_TOTAL_SIZE, tulip_tbl[chip_id].chip_name);

    dev->base_addr = ioaddr;
    dev->irq = irq;

    // kmalloc(sizeof(*tp),GFP_KERNEL|GFP_DMA)
    /* Make certain the data structures are quadword aligned. */
    tpadapter = (void *)(((int)kmalloc(sizeof(*tpadapter), GFP_KERNEL | GFP_DMA) + 7) & ~7);
    memset(tpadapter,0,sizeof(*tpadapter));
    tpadapter->Pub.Port=ioaddr+CSR9;
    dev->priv = tpadapter;


#ifdef  MODULE
    tpadapter->next_module = root_dlkpci_dev;
    root_dlkpci_dev = dev;
#endif
    tpadapter->dwCFID = chip_id;


    /* The Tulip-specific entries in the device structure. */
    dev->open = &tulip_open;
    dev->hard_start_xmit = &tulip_start_xmit;
    dev->stop = &tulip_close;
    dev->get_stats = &tulip_get_stats;
#ifdef HAVE_MULTICAST
    dev->set_multicast_list = &set_multicast_list;
#endif

    // parse srom  --yet not implemented--becker

    /* if options is non-zero value, user specified media type. */
    tpadapter->default_media = options % MAX_MEDIA_TABLE;
    if (options!=AUTOSENSE)
        printk("Using user specified media : %s\n",force_media_tbl[options].media_type_mesg);
 
    switch (chip_id) {
    case   DC21140:
           outl(0x100,ioaddr+CSR12);
           break;
    case   DC21143:
    //       outl(0x100,ioaddr+CSR12);
           break;
    case   MX98713:
           outl(0x100,ioaddr+CSR12);
           break;
    }

    return dev;

} /* tulip_probe1()*/


static int tulip_open( struct device *dev)
{
    tADAPTER  *tpadapter = (tADAPTER *)dev->priv;
    int ioaddr = dev->base_addr;
    
    /* fixed the bug of DC21140 caused TX hang at the first time to used it */ 
    /* On some chip revs we must set the MII/SYM port before the reset!? */
    if (tpadapter->dwCFID==DC21140)
        outl(0x00040000,ioaddr + CSR6);

    /* Software Reset by writting CSR0<0> to 1 */
    outl(1,ioaddr + CSR0);

#ifdef _LINUX_DELAY_H
        udelay(2);
#else
        SLOW_DOWN_IO;
#endif
    if (tpadapter->dwCFID==MX98713){/*Writting CSR16 after software reset */
        outl(0x0f7ffec8,ioaddr+CSR16);
        outl(0x01cc0000,ioaddr+CSR6);
    }/*if*/

#if defined(MODULE)
    /* When a module we don't have 'x86' to check. */
    //outl(0x00200000 | 0x4800 | 0x0008, ioaddr + CSR0);
    //outl(0xfe00e010,ioaddr +CSR0);
      outl(0xfe008110,ioaddr + CSR0);
#else
    outl(0x00200000 |0x0008| (x86 <= 4 ? 0x4800 : 0x8000), ioaddr + CSR0);
    if (x86 <= 4)
        printk("This is a 386/486 PCI system, setting cache alignment to %x.\n",
                0x00200000 | (x86 <= 4 ? 0x4800 : 0x8000));
#endif  /* MODULE */

#ifdef SA_SHIRQ
    if (request_irq(dev->irq, &tulip_interrupt, SA_SHIRQ,
                    tulip_tbl[tpadapter->dwCFID].chip_name, dev)) {
        return -EAGAIN;
    } /* if */
#else
    if (irq2dev_map[dev->irq] != NULL
        || (irq2dev_map[dev->irq] = dev) == NULL
        || dev->irq == 0
        || request_irq(dev->irq, &tulip_interrupt, 0,
        tulip_tbl[tpadapter->dwCFID].chip_name)) {
            return -EAGAIN;
    } /* if */
#endif
    // initial Rx/Tx rings
    tulip_init_ring(dev);

    /* initialized management pointers */
    tpadapter->pEnqueueTxDesc = tpadapter->pTxRing;
    tpadapter->pDequeueTxDesc = tpadapter->pTxRing;
    tpadapter->pDequeueRxDesc = tpadapter->pRxRing;
    tpadapter->pFreeRxBuffer = RX_RING_SIZE;
    tpadapter->nNumUnusedTxDesc = TX_RING_SIZE;

    // set packet filtering by sending Setup-Frame
    {
        unsigned short *eaddrs= (unsigned short*)dev->dev_addr;
        int *setup_frm = (int *)tpadapter->SetupBuffer,i;

        /* Added broadcast address to our address filter */
        *setup_frm++ = 0xffff;
        *setup_frm++ = 0xffff;
        *setup_frm++ = 0xffff;
        
        for (i=1; i<16;i++) {
            *setup_frm++ = eaddrs[0];
            *setup_frm++ = eaddrs[1];
            *setup_frm++ = eaddrs[2];
        } /* for */

        tpadapter->pEnqueueTxDesc->wOwnBit = 0x8000;
        tpadapter->pEnqueueTxDesc->wStatus = 0x0000;
        tpadapter->pEnqueueTxDesc->dwCommand = 0x08000000 | 192|0x01000000;
        tpadapter->pEnqueueTxDesc->dwAddress1= virt_to_bus(tpadapter->SetupBuffer);

        /* updated managed pointers */
        tpadapter->nNumUnusedTxDesc--;
        tpadapter->pEnqueueTxDesc = tpadapter->pEnqueueTxDesc->pNext;

    }

    outl(virt_to_bus(tpadapter->pRxRing), ioaddr + CSR3);
    outl(virt_to_bus(tpadapter->pTxRing), ioaddr + CSR4);

    /* Detecting the current media type */
    if (tpadapter->default_media)
        set_force_media(dev);
    else
        select_media(dev);

    /* start chip's TX to send setup frame */
    DBG_CSR6("tulip_open(): csr6=%#8.8x\n",(u32)tpadapter->csr6);
    outl(tpadapter->csr6, ioaddr + CSR6);
    outl(tpadapter->csr6 | 0x2000, ioaddr + CSR6);      /* start TX command */

    dev->tbusy = 0;
    dev->interrupt = 0;
    dev->start = 1;
    outl(tpadapter->csr6 | 0x2002, ioaddr + CSR6);
    
    /* Enable interrupts by setting the interrupt mask. */
    outl(tpadapter->csr7,ioaddr +CSR7);
 
    outl(0, ioaddr + CSR2);         /* Rx poll demand */

    /* If user specify media type, set the timer to switch to check for 
       link beat and perhaps switch to an alternate media type */
    if (tpadapter->default_media == AUTOSENSE){
        init_timer(&tpadapter->timer);
        tpadapter->timer.expires = RUN_AT((24*HZ)/10);     /* 2.4 sec. */
        tpadapter->timer.data = (unsigned long)dev;
        tpadapter->timer.function = &tulip_timer;          /* timer handler */
        add_timer(&tpadapter->timer);
    }
    return 0;
} /* tulip_open() */

static void set_force_media(struct device *dev)
{
    tADAPTER     *tpadapter = (tADAPTER *)dev->priv;
    int          ioaddr = dev->base_addr; 

    tpadapter->MediaCtrlBlk.MiiMediaType.cConnection = force_media_tbl[tpadapter->default_media].cConnection; 
    switch (tpadapter->dwCFID){
         case DC21140: case MX98713:
              bMacToPhyInit(&tpadapter->Pub,&tpadapter->MediaCtrlBlk,tpadapter->default_media);
              bMacSetPhyControl(&tpadapter->Pub,&tpadapter->MediaCtrlBlk);
              showmedia_change_csr6(tpadapter);
              break;

         case DC21143:{
              switch (tpadapter->default_media){
                  case F10BASET:     
                  case F10BASET_FD:
                       outl(inl(ioaddr + CSR6)&~0x2002,ioaddr + CSR6);
                       outl(0,ioaddr+CSR13);
                       outl(0xff3f,ioaddr+CSR14);
                       outl(0x0008,ioaddr+CSR15);
                       outl(1,ioaddr+CSR13);
                       break;
                  case F100BASETX:
                  case F100BASETX_FD:
                       outl(inl(ioaddr+CSR6)&~0x2002,ioaddr+CSR6);
                       outl(0,ioaddr+CSR13);
                       outl(0xfff80000,ioaddr+CSR14);
                       outl(0x0008,ioaddr+CSR15);
                       outl(1,ioaddr+CSR13);
                       break;
              }/*switch*/
              showmedia_change_csr6(tpadapter);
              tpadapter->csr7 &= ~0x08001010; /* disable LC LP & ANC interrupt */
              break;
         }/*DC21143*/
    }/*switch*/

    

}/*force_media()*/

static void select_media(struct device *dev)
{
    tADAPTER     *tpadapter = (tADAPTER *)dev->priv;
    int          wCType=CT_NWAY_AUTO;
    tMII_PHY_INFO  phy_info;

    switch (tpadapter->dwCFID){
         case DC21140:case MX98713:{
              /* Now we supported MII version only. Find MII-PHY */
              /* mac2phy.c */
              printk("Initializating MII...");
              bMacToPhyInit (&tpadapter->Pub,&tpadapter->MediaCtrlBlk, wCType);
              printk("Done.\n");
              printk("Detecting Connection Type: ");

              phy_info = tpadapter->MediaCtrlBlk.PhyInfo;
              if (phy_info.dwPhyID==PHYID_DP83840)
                  check_dfe500c1_connection(tpadapter,dev->base_addr);
              else
                  if (bDetectMediaType(&tpadapter->Pub,&tpadapter->MediaCtrlBlk)==eFALSE)
                      tpadapter->MediaCtrlBlk.MiiMediaType.cConnection = 0xff; /* no connection*/

              showmedia_change_csr6(tpadapter);
              break;
         }/* DC21140*/
         case DC21143:{
             detect_21143_media(dev);
             showmedia_change_csr6(tpadapter);
         }/*DC21143*/
              break;
    } /*switch*/
    return;
}


void detect_21143_media(struct device *dev)
{
     int ioaddr = dev->base_addr;
     int i=0;
     tADAPTER     *tpadapter = (tADAPTER *)dev->priv;
     unsigned char *pAnc_ok = &anc_ok; /* get rid of gcc compiler's bug */

     printk("Starting to detect DC21143 media type...\n");
//     outl(0x3ff80001,ioaddr+CSR0); /*Reset the chip */
       outl(inl(ioaddr+CSR6)& ~0x2002,ioaddr+CSR6); /* Stop TX/RX */
#ifdef _LINUX_DELAY_H
    udelay(2);
#else
    SLOW_DOWN_IO;
#endif
     *pAnc_ok=AUTOSENSE_FAIL;
     link_change_intr=0;
#if 0
     outl(0,ioaddr+CSR13);  /* Reset SIA */
     outl(0x32c02202,ioaddr+CSR6);
     outl(0xfffbf7fd,ioaddr+CSR14);
     outl(0x1,ioaddr+CSR13); /* Start SIA */
#endif 
     outl(0x32c02002,ioaddr +CSR6);
     outl(0,ioaddr+CSR13);
     outl(0xfffbF7FD,ioaddr+CSR14);
     outl(1,ioaddr +CSR13); 
     outl(0xfbff9bff,ioaddr +CSR7);
     DBG_CSR7("detect_media(): csr7=%#8.8x\n",inl (ioaddr+CSR7));

     while ((link_change_intr<LINK_CHANGE_INTERRUPT_MAX) && (i++<1000))
        if(*pAnc_ok==AUTOSENSE_SUCCESS) 
           break;
        else
           udelay(3000);

     if (*pAnc_ok==AUTOSENSE_SUCCESS)
        tpadapter->dc21143_pre_connection_type=tpadapter->MediaCtrlBlk.MiiMediaType.cConnection;
     else{
        tpadapter->dc21143_pre_connection_type=0xff; /* No connection */
        tpadapter->MediaCtrlBlk.MiiMediaType.cConnection=0xff;
     }

}/* detect_21143_media() */

static void dc21143_link_change(struct device *dev,int csr5)
{
    int ioaddr = dev->base_addr;

    tADAPTER     *tpadapter = (tADAPTER *)dev->priv;

    link_change_intr++;

    if ((csr5 & tpadapter->csr7 & 0x08000000)&&anc_ok==AUTOSENSE_DYNAMIC){
        DBG_LINK_CHANGE_INTR("Link change interrupt(100M)....\n");
        anc_ok= AUTOSENSE_AGAIN; /* do dynamically autosense later */
 ;;       return;
    }
    if ((csr5 & tpadapter->csr7 & 0x00001000)&& anc_ok==AUTOSENSE_DYNAMIC){
        DBG_LINK_CHANGE_INTR("Link Fail(10M)....\n");
        anc_ok = AUTOSENSE_AGAIN; /* do dynamicall autosense later */
 ;;       return;
    }
    if (csr5& 0x00000010) {/* Auto Negotiation Complete CSR5<4> */
        int csr12;
        anc_ok =AUTOSENSE_SUCCESS;
        outl(csr5&0x08001010,ioaddr+CSR5);

        printk("Auto Negotiation Complete...");
        csr12 = inl(ioaddr+CSR12);
        DBG_CSR12("dc21143_link_change(): csr12=%#8.8x\n",csr12);
        if ((csr12&0x7000)==0x5000) { /* autonegotiation complete */
            if (csr12 &0x8000) { /* Link partner Negotiable */
                outl(0,ioaddr + CSR6);
                outl(0,ioaddr + CSR13);
                outl(0xfff80000,ioaddr + CSR14);
                outl(0x08a00008,ioaddr + CSR15);
                outl(0x1,ioaddr + CSR13);
                outl(0x08000000,ioaddr + CSR15); /* direction */
                outl(0x0008,ioaddr + CSR15);    /* data */
                tpadapter->MediaCtrlBlk.MiiMediaType.cConnection=MII_100BASE_TX_FD;
                DBG_CSR12("dc21143_link_change(): 100Mbps FULL duplex Negotiable..\n");
            }else{
                if (!(csr12 & 0x00000002)){/* 100M */
                    outl(0,ioaddr + CSR6);
                    outl(0,ioaddr + CSR13);
                    outl(0xfff80000,ioaddr + CSR14);
                    outl(0x0008, ioaddr + CSR15);
                    if (csr12 & 0x00008000){  /* 100M FD */
                       tpadapter->MediaCtrlBlk.MiiMediaType.cConnection=MII_100BASE_TX_FD;
                       DBG_CSR12("dc21143_link_change(): 100Mbps FULL duplex..\n");
                    } else{/* 100M HD */
                       tpadapter->MediaCtrlBlk.MiiMediaType.cConnection=MII_100BASE_TX;
                       DBG_CSR12("dc21143_link_change(): 100Mbps HALF duplex..\n");
                    }
                    /* if */
                 }else { /*10M*/
                     if (!(csr12 & 0x00000004)){/*10M */
                         outl(0,ioaddr + CSR6);
                         outl(0,ioaddr+CSR13); /* reset SIA */
                         outl(0xff3f,ioaddr+CSR14);
                         outl(0x0008,ioaddr + CSR15);
                         outl(0x1,ioaddr + CSR13);
                         if (csr12 & 0x00008000){
                             tpadapter->MediaCtrlBlk.MiiMediaType.cConnection=MII_10BASE_T_FD;
                             DBG_CSR12("dc21143_link_change(): 10Mbps FULL duplex..\n");
                         }else{
                             tpadapter->MediaCtrlBlk.MiiMediaType.cConnection=MII_10BASE_T;
                             DBG_CSR12("dc21143_link_change(): 10Mbps half duplex..\n");
                         }
                     }
                }
           }
         }
        printk("Done..\n");
       }
} /* dc21143_link_change() */

static void check_dfe500c1_connection(tPADAPTER tpadapter,int ioaddr)
{
    tCSR12_BITS CSR12Value;
    tBYTE       connection_type;

    outl(0x100,ioaddr+CSR12);
#ifdef _LINUX_DELAY_H
    udelay(2);
#else
    SLOW_DOWN_IO;
#endif

        CSR12Value.lw = inl(ioaddr + CSR12);
        connection_type = csr12_to_connection_type(CSR12Value);

        tpadapter->MediaCtrlBlk.MiiMediaType.cConnection = connection_type;
        tpadapter->OldCSR12Value = CSR12Value;

} /*check_dfe500c1_connection() */

static tBYTE csr12_to_connection_type(tCSR12_BITS CSR12Value)
{
  tBYTE connection_type;

        if (CSR12Value.bits.link_status)  /* no connection */
            connection_type=0xff; /* no connection */
        else
        {
            if (CSR12Value.bits.duplex_mode)  /* half duplex */
            {
                if (CSR12Value.bits.line_speed) /* 100Mbps */
                    connection_type = MII_100BASE_TX;
                else     /* 10 Mbps */
                    connection_type = MII_10BASE_T;
            }
            else     /* full duplex */
            {
                if (CSR12Value.bits.line_speed) /* 100Mbps */
                    connection_type = MII_100BASE_TX_FD;
                else     /* 10 Mbps */
                    connection_type = MII_10BASE_T_FD;
            }
        }

   return connection_type;
} /* csr12_to_connection_type() */


static void showmedia_change_csr6(tPADAPTER tpadapter)
{
    tBYTE   connection_type= tpadapter->MediaCtrlBlk.MiiMediaType.cConnection;
    switch (connection_type){

        case MII_100BASE_TX_FD:
             if (tpadapter->dwCFID==DC21143){
                  tpadapter->csr6 = 0x338c0000|0x0200;
                  tpadapter->csr7 = 0xfbff9079;
             }else{
                  tpadapter->csr6 = tpadapter->csr6=0x320c0000|0x0200;
                  tpadapter->csr7 = 0x0001fbef;
             }
             printk("100MBps Full Duplex.\n");
             break;
        case MII_100BASE_TX:
             if (tpadapter->dwCFID==DC21143){
                  tpadapter->csr6 = 0x338c0000;
            //      tpadapter->csr7 = 0xfbff9050;/* TUE<2> must be disable*/
                  tpadapter->csr7 = 0xfbffb079;
             }else{
                  tpadapter->csr6 = tpadapter->csr6=0x320c0000;
                  tpadapter->csr7 = 0x0001fbef;
             }
             printk("100MBps Half Duplex.\n");
             break;
        case MII_10BASE_T_FD:
             if (tpadapter->dwCFID==DC21143){
                tpadapter->csr6=0x32c00000 |0x0200;
                tpadapter->csr7=0xf3ff9079;
             }else{
                tpadapter->csr6=0x324c0000|0x0200;
                tpadapter->csr7=0x0001fbef;
             }
             printk("10MBps Full Duplex.\n");
             break;
        case MII_10BASE_T:
             if (tpadapter->dwCFID==DC21143){
                 tpadapter->csr6=0x32c00000;
                 tpadapter->csr7=0xf3ff9079;
             }else {
                 tpadapter->csr6=0x324c0000;
             //    tpadapter->csr7=0x0001fbef;
                 tpadapter->csr7 = 0x00018069;
             }
             printk("10MBps Half Duplex.\n");
             break;
        case 0xff: /* conncetion not found */
             if (tpadapter->dwCFID==DC21143){
                 tpadapter->csr6=0x32c00000;
                 tpadapter->csr7=0xf3ff9051;
             }else {
                 tpadapter->csr6=0x324c0000;
                 tpadapter->csr7=0x0001fbef;
             }

             printk("No connection !!\n");
             break;
        default:
             break;

    } /* switch */
} /* showmedia_change_csr6() */


static void tulip_init_ring(struct device *dev)
{
    tADAPTER  *tpadapter = (tADAPTER *)dev->priv;
    tTX_DESC  *tx_desc;
    tRX_DESC  *rx_desc;
    int i;

    /* allocated TX_DESC and initialized TX descriptors' link list */
    tx_desc = (void *)(((long)kmalloc(sizeof(tTX_DESC), GFP_KERNEL | GFP_DMA) + 7) & ~7);
    tpadapter->pTxRing=tx_desc;

    for (i=1;i<TX_RING_SIZE; i++){
        tx_desc->pNext=(void*)(((long)kmalloc(sizeof(tTX_DESC),GFP_KERNEL|GFP_DMA)+7)&~7);
        tx_desc = tx_desc->pNext;
    }
    tx_desc->pNext=tpadapter->pTxRing;

    /* allocated RX_DESC and initialized RX descriptors' link list */
    rx_desc = (void *)(((long)kmalloc(sizeof(tRX_DESC), GFP_KERNEL | GFP_DMA) + 7) & ~7);
    tpadapter->pRxRing = rx_desc;

    for (i=1;i<RX_RING_SIZE; i++){
        rx_desc->pNext=(void*)(((long)kmalloc(sizeof(tRX_DESC),GFP_KERNEL|GFP_DMA)+7)&~7);
        rx_desc=rx_desc->pNext;
    }    
    rx_desc->pNext=tpadapter->pRxRing;

    /* TX Descriptors initialization */
    tx_desc = tpadapter->pTxRing;
    for (i=1;i <= TX_RING_SIZE;i++){
        tx_desc->wStatus = 0x0000;
        tx_desc->wOwnBit = 0x0000;
        tx_desc->dwCommand = 0x01000000; /* second address is chained */
        tx_desc->tx_skbuff = 0;
        tx_desc->dwAddress2 = virt_to_bus(tx_desc->pNext);
        if (i != TX_RING_SIZE)  /* pinoter is not updated when last */
            tx_desc = tx_desc->pNext;
    }/* for */
    tx_desc->dwAddress2 = virt_to_bus(tpadapter->pTxRing);

    /* RX Descriptors initialization */
    rx_desc = tpadapter->pRxRing;
    for (i=1; i<= RX_RING_SIZE; i++){
        rx_desc->wFrameLength = 0x8000; /* Owned by Tulip chip */
        rx_desc->wStatus = 0x0000;
        rx_desc->dwCommand = PKT_BUF_SZ|0x01000000;  /* chain structure */
        {
            /* Note the receive buffer must be longword aligned.
               dev_alloc_skb() provides 16 byte alignment.  But do *not*
               use skb_reserve() to align the IP header! */
            struct sk_buff *skb;
            skb = DEV_ALLOC_SKB(PKT_BUF_SZ);
            rx_desc->rx_skbuff = skb;
            if (skb == NULL)
                break;                  /* Bad news!  */
            skb->dev = dev;                 /* Mark as being used by this device. */
#if LINUX_VERSION_CODE > 0x10300
            rx_desc->dwAddress1 = virt_to_bus(skb->tail);
#else
            rx_desc->dwAddress1 = virt_to_bus(skb->data);
#endif
         }
         rx_desc->dwAddress2 = virt_to_bus(rx_desc->pNext);
         if (i!= RX_RING_SIZE) /* pointer is not updated when last */
            rx_desc = rx_desc->pNext;
    }/* for */

    /* Mark the last entry as wrapping the ring. */
    rx_desc->dwCommand = 0x01000000| PKT_BUF_SZ; /* second address is chained  */
    rx_desc->dwAddress2 = virt_to_bus(tpadapter->pRxRing);

}/* tulip_init_ring() */



static void tulip_timer(unsigned long data)
{
     struct device *dev = (struct device *)data;
     tADAPTER *tp = (tADAPTER *)dev->priv;
     int next_tick = 0;


    //if media is changed, reading MiiPhy(BMSR) to change mediatype.
    if (tp->MediaCtrlBlk.MiiMediaType.cConnection ==0xff)
       printk("No connection !!\n");
    switch (tp->dwCFID){
       case MX98713:
       case DC21140:{
            if (tp->MediaCtrlBlk.PhyInfo.dwPhyID==PHYID_DP83840){
                tCSR12_BITS CSR12Value;
                tBYTE       connection_type;

                CSR12Value.lw = inl(dev->base_addr + CSR12);
                connection_type = csr12_to_connection_type(CSR12Value);
                if (tp->MediaCtrlBlk.MiiMediaType.cConnection != connection_type){
                    printk("Media has changed. ");
                    printk("Switching to ");
                    tp->MediaCtrlBlk.MiiMediaType.cConnection = connection_type;
                    showmedia_change_csr6(tp);
                    tp->OldCSR12Value = CSR12Value;
                    /* Stop the chip's Tx and Rx processes. */
                    outl(inl(dev->base_addr + CSR6) & ~0x2002, dev->base_addr + CSR6);
                    outl(tp->csr6|0x2002,dev->base_addr+CSR6);
                    outl(tp->csr7,dev->base_addr+CSR7);
               }

          } else{
              tBOOLEAN pLineSpeed100,pFDEnable;
              tMEDIA_CTRL    MediaCtrlBlk=tp->MediaCtrlBlk;
              MediaCtrlBlk.MiiMediaType.cControl= MII_NWAY_AUTOSENSE;
              if (DynamicSenseMedia(&tp->Pub,&MediaCtrlBlk, &pLineSpeed100,&pFDEnable)){
                 if (tp->MediaCtrlBlk.MiiMediaType.cConnection !=
                     MediaCtrlBlk.MiiMediaType.cConnection){
                         printk("Media has changed. ");
                         printk("Switching to ");
                         tp->MediaCtrlBlk=MediaCtrlBlk;
                         showmedia_change_csr6(tp);
                         /* Stop the chip's Tx and Rx processes. */
                         outl(inl(dev->base_addr + CSR6) & ~0x2002, dev->base_addr + CSR6);
                         outl(tp->csr6|0x2002,dev->base_addr+CSR6);
                         outl(tp->csr7,dev->base_addr+CSR7);
                  }
             } else{
                 printk("Media has changed. No connection !!\n");
                 tp->MediaCtrlBlk.MiiMediaType.cConnection=0xff; /*no connection */
             }
          }
       }/*DC21140*/
       break;
       case DC21143:{
             if (dev->interrupt) {
                 printk("timer in tulip_interupt..\n");
                 break;
             }
             if (anc_ok==AUTOSENSE_AGAIN){
                 detect_21143_media(dev);
                 if (anc_ok==AUTOSENSE_SUCCESS){
                       showmedia_change_csr6(tp);
                       /* Stop the chip's Tx and Rx processes. */
                       outl(inl(dev->base_addr + CSR6) & ~0x2002, dev->base_addr +CSR6);
                       outl(tp->csr6|0x2002,dev->base_addr+CSR6);
                       outl(tp->csr7,dev->base_addr+CSR7);
                      DBG_CSR6("tulip_timer(): csr6=%#8.8x\n",(u32)tp->csr6);
                 }else
                     anc_ok = AUTOSENSE_AGAIN; /* Autosense is not successed,running again */
             }else
                 anc_ok = AUTOSENSE_DYNAMIC;
       }/*DC21143*/
       break;
    }/*switch */


    next_tick = (24*HZ)/10;
    if (next_tick) {
        tp->timer.expires = RUN_AT(next_tick);
        add_timer(&tp->timer);
    } /* if */


} /* tulip_timer() */


static void tulip_tx_timeout(struct device *dev)
{
    tADAPTER    *tpadapter = (tADAPTER *) dev->priv;
    int         ioaddr = dev->base_addr;
    /* Stop the transmit process. */
    outl(inl(ioaddr + CSR6) & (~0x2000),ioaddr + CSR6);
    printk("%s: 2114x transmit timed out.\n",dev->name); 
    outl(tpadapter->csr6|0x2002,ioaddr + CSR6);
    tpadapter->stats.tx_errors++;
    dev->trans_start = jiffies;
  
    return; 

} /* tulip_tx_timeout() */

static int tulip_rx(struct device *dev)
{
    tADAPTER    *tpadapter = (tADAPTER *)dev->priv;
    tPRX_DESC   rx_desc;
    DBG_RX_INTR("tulip_rx()....\n");
    rx_desc = tpadapter->pDequeueRxDesc;
    DBG_RX_INTR("tulip_rx(): rx_desc->wFrameLength--[%4.4x]\n",rx_desc->wFrameLength);
#if 0
    { int i;tPRX_DESC rx_desc1;
      rx_desc1= tpadapter->pRxRing;
      printk("\n=========================\n");
      printk("rx_desc->wFrameLength:\n\n");
      for (i=1;i<=RX_RING_SIZE;i++,rx_desc1=rx_desc1->pNext) {
          printk("%04x ",(u16)rx_desc1->wFrameLength);
          if ((i%4)==0) printk("\n"); 
      }/*for*/ 
      printk("\n=========================\n");
    }/* temp*/
#endif /* 0 */

    while(rx_desc->wFrameLength >= 0){
        tUSHORT rx_status = rx_desc->wStatus;
        DBG_RX_INTR("tulip_rx(): rx_desc--[%8.8x]\n",rx_desc);
        DBG_RX_INTR("tulip_rx(): rx_status--[%4.4x]\n",rx_status);

        if ((rx_status & (RDES0_FS|RDES0_LS)) != (RDES0_FS|RDES0_LS)) {
            if ((rx_status & 0xffff) != 0x7fff) { /* Ingore earlier buffers. */
                printk("%s: Oversized Ethernet frame spanned multiple buffers,"
                       " status %8.8x!\n", dev->name, rx_status);
                tpadapter->stats.rx_length_errors++;
            }
        } else if (rx_status & RDES0_ES) { /* error summary */
                    /* There was a fatal error. */
                    tpadapter->stats.rx_errors++; /* end of a packet.*/
                    if (rx_status & (RDES0_RF |RDES0_TL |RDES0_RW))
                        tpadapter->stats.rx_length_errors++;
                    if (rx_status & RDES0_DB) tpadapter->stats.rx_frame_errors++;
                    if (rx_status & RDES0_CE) tpadapter->stats.rx_crc_errors++;
                    if (rx_status & RDES0_OF) tpadapter->stats.rx_fifo_errors++;
                } else {
                    /* Malloc up new buffer, compatible with net-2e. */
                    /* Omit the four octet CRC from the length. */
                    short pkt_len = (rx_desc->wFrameLength &0x3fff) - 4;
                    struct sk_buff *skb;
                    int rx_in_place = 0;
                    DBG_RX_INTR("tulip_rx(): RX packet length--[%4.4d]\n", pkt_len);
                    /* Check if the packet is long enough to just accept without
                       copying to a properly sized skbuff. */
                    if (pkt_len > SKBUFF_RX_COPYBREAK) {
                        struct sk_buff *newskb;
                        char *temp;

                        /* Get a fresh skbuff to replace the filled one. */
                        newskb = DEV_ALLOC_SKB(PKT_BUF_SZ);
                        if (newskb == NULL) {
                                skb = 0;                /* No memory, drop the packet. */
                                goto memory_squeeze;
                        }
                        /* Pass up the skb already on the Rx ring. */
                        skb = rx_desc->rx_skbuff;
                        temp = skb_put(skb, pkt_len);
                        if (bus_to_virt(rx_desc->dwAddress1) != temp)
                            printk("%s: Warning -- the skbuff addresses do not match"
                                   " in tulip_rx: %p vs. %p / %p.\n", dev->name,
                                    bus_to_virt(rx_desc->dwAddress1), skb->head, temp);

                        rx_in_place = 1;
                        rx_desc->rx_skbuff = newskb;
                        newskb->dev = dev;
                        /* Longword alignment required: do not skb_reserve(2)! */
                        rx_desc->dwAddress1 = virt_to_bus(newskb->tail);
                    } else
                        skb = DEV_ALLOC_SKB(pkt_len + 2);

                    memory_squeeze:
                    if (skb == NULL) {
                        int i;
                        tPRX_DESC rx_desc1 = rx_desc;
                        printk("%s: Memory squeeze, deferring packet.\n", dev->name);
                        /* Check that at least two ring entries are free.
                           If not, free one and mark stats->rx_dropped++. */

                        for (i = 0 ; i < RX_RING_SIZE; i++)
                            if (rx_desc1->wFrameLength < 0)
                                break;
                            else
                                rx_desc1 = rx_desc1->pNext;

                        if (i > RX_RING_SIZE -2) {
                            tpadapter->stats.rx_dropped++;
                            rx_desc1->wStatus = 0x8000; /* dropped this packet */
                        ////????    rx_desc1 = rx_desc1->pNext;
                        }/* if */
                        break; /* out of while() */
                    } /* if */
                    skb->dev = dev;
                    if (! rx_in_place) {
                        skb_reserve(skb, 2);    /* 16 byte align the data fields */
                        memcpy(skb_put(skb, pkt_len), bus_to_virt(rx_desc->dwAddress1), pkt_len);
                    }
#if LINUX_VERSION_CODE > 0x10300
                    skb->protocol = eth_type_trans(skb, dev);
#else
                    skb->len = pkt_len;
#endif
                    netif_rx(skb);
                    DBG_RX_INTR("tulip_rx(): netif_rx()....\n");
                    tpadapter->stats.rx_packets++;
                } /* if-else */

                rx_desc->wFrameLength = 0x8000;
                rx_desc=rx_desc->pNext;
                DBG_RX_INTR("tulip_rx(): rx_desc->wFrameLength--[%4.4x]\n",rx_desc->wFrameLength);

    } /* while() */

    tpadapter->pDequeueRxDesc = rx_desc; /* updated pointer */

    return 0;
} /* tulip_rx() */


static int
tulip_start_xmit(struct sk_buff *skb, struct device *dev)
{

    tADAPTER    *tpadapter = (tADAPTER *)dev->priv;
    tPTX_DESC   tx_desc;
    u32 flag;
#ifndef final_version
    if (skb == NULL || skb->len <= 0) {
        printk("%s: Obsolete driver layer request made: skbuff==NULL.\n", dev->name);
        dev_tint(dev);
        return 0;
    }
#endif

    /* Block a timer-based transmit from overlapping.  This could better be
       done with atomic_swap(1, dev->tbusy), but set_bit() works as well. */
    if (set_bit(0, (void*)&dev->tbusy) != 0) {
        if (jiffies - dev->trans_start < TX_TIMEOUT)
            return 1;
        tulip_tx_timeout(dev);
        return 1;
    }
    /* Caution: the write order is important here, set the base address
       with the "ownership" bits last. */

    tx_desc = tpadapter->pEnqueueTxDesc;


//    printk("{tulip_start_xmit(): tx_desc--[%8.8x]} \n",(u32)tx_desc);
//    printk("{tulip_start_xmit(): nNumUnusedTxDesc--[%4.4d]\n",(u16)tpadapter->nNumUnusedTxDesc);

    tpadapter->pEnqueueTxDesc = tx_desc->pNext; /* point to next descriptor */
    if (tpadapter->nNumUnusedTxDesc>0)
        tpadapter->nNumUnusedTxDesc--;

    tx_desc->tx_skbuff = skb;
    tx_desc->dwAddress1 = virt_to_bus(skb->data);

    if ( tpadapter->nNumUnusedTxDesc > TX_RING_SIZE/2) {/* Typical path */
        flag = 0x61000000; /* No interrupt */
        dev->tbusy = 0;
    } else
        if (tpadapter->nNumUnusedTxDesc == TX_RING_SIZE/2) {
        flag = 0xe1000000; /* Tx-done intr. */
        dev->tbusy = 0;
    } else if (tpadapter->nNumUnusedTxDesc > TX_RING_SIZE - 2) {
      flag = 0x61000000; /* No Tx-done intr. */
      dev->tbusy = 0;
    } else {
      /* Leave room for set_rx_mode() to fill entries. */
      flag = 0xe1000000; /* Tx-done intr. */
      tpadapter->tx_full = 1;
    }

    tx_desc->dwCommand = skb->len | flag;
    tx_desc->wOwnBit = 0x8000; /* Pass ownership to the chip. */
    tx_desc->wStatus = 0x0000;

    /* Trigger an immediate transmit demand. */
    outl(0, dev->base_addr + CSR1);

    dev->trans_start = jiffies;

    return 0;
}
/* The interrupt handler does all of the Rx thread work and cleans up
   after the Tx thread. */
static void tulip_interrupt IRQ(int irq, void *dev_instance, struct pt_regs *regs)
{
#ifdef SA_SHIRQ         /* Use the now-standard shared IRQ implementation. */
    struct device *dev = (struct device *)dev_instance;
#else
    struct device *dev = (struct device *)(irq2dev_map[irq]);
#endif

    tADAPTER    *tpadapter = (tADAPTER *)dev->priv;
    tPTX_DESC   pEnqueueTxDesc,pDequeueTxDesc;

    int csr5, ioaddr, boguscnt = 10;
    if (dev == NULL) {
        printk ("tulip_interrupt(): irq %d for unknown device.\n", irq);
        return;
    }

    ioaddr = dev->base_addr;
    if (dev->interrupt)
        printk("%s: Re-entering the interrupt handler.\n", dev->name);

    dev->interrupt = 1;

    do {
        csr5 = inl(ioaddr + CSR5);
        /* Acknowledge all of the current interrupt sources ASAP. */
          outl(csr5, ioaddr + CSR5);

         DBG_CSR5("%s: interrupt  csr5=%#8.8x new csr5=%#8.8x.\n",
                   dev->name, csr5, inl(dev->base_addr + CSR5));
         DBG_CSR6("%s: tulip_interrupt() csr6=%#8.8x.\n",dev->name,inl(dev->base_addr+CSR6));
        if ((csr5 & 0x08001010) && tpadapter->dwCFID==DC21143&&
            tpadapter->default_media==AUTOSENSE) {/* Link change or Link fail */
           dc21143_link_change(dev,csr5);
           break;
        }

        if ((csr5 & 0x00018000) == 0) /* no abnormal & Normal Interrupt */
            break;


        if (csr5 & 0x0040)  /* Rx interrupt */
            tulip_rx(dev);

        if (csr5 & 0x0007) {/* Tx-done interrupt */
            pDequeueTxDesc = tpadapter->pDequeueTxDesc;
            pEnqueueTxDesc = tpadapter->pEnqueueTxDesc;

            while (pDequeueTxDesc != pEnqueueTxDesc){
                short status = pDequeueTxDesc->wStatus;
                //
                DBG_TX_DESC("tulip_interrupt():Tx-Done:status-[%4.4x]..\n",(u16)pDequeueTxDesc->wStatus);
                DBG_TX_DESC("tulip_interrupt():Tx-Done:DequeueTxDesc-[%8.8x]..",(u32)pDequeueTxDesc);
                DBG_TX_DESC("tulip_interrupt():Tx-Done:UnusedTxDesc--[%4.4d]..\n",tpadapter->nNumUnusedTxDesc);
                //

                if (pDequeueTxDesc->wOwnBit & 0x8000)
                    break; /* It still hasn't been Txed */

                /* Check for Rx filter setup frames. */
                if (pDequeueTxDesc->tx_skbuff == NULL){
                    pDequeueTxDesc=pDequeueTxDesc->pNext;
                    tpadapter->nNumUnusedTxDesc++;
                    continue;
                }/*if*/

                if (status & 0x8000) {  /* Error Summary */
                    /* There was an major error, log it. */
#ifndef final_version
                    if (tulip_debug > 1)
                        printk("%s: Transmit error, Tx status %8.8x.\n", dev->name, status);
#endif
                    tpadapter->stats.tx_errors++;
                    if (status & 0x4104) tpadapter->stats.tx_aborted_errors++;
                    if (status & 0x0C00) tpadapter->stats.tx_carrier_errors++;
                    if (status & 0x0200) tpadapter->stats.tx_window_errors++;
                    if (status & 0x0002) tpadapter->stats.tx_fifo_errors++;
                    if ((status & 0x0080) && tpadapter->tx_full_duplex == 0)
                        tpadapter->stats.tx_heartbeat_errors++;
#ifdef ETHER_STATS
                    if (status & 0x0100) tpadapter->stats.collisions16++;
#endif
                } else {
#ifdef ETHER_STATS
                    if (status & 0x0001) tpadapter->stats.tx_deferred++;
#endif
                    tpadapter->stats.collisions += (status >> 3) & 15;
                    tpadapter->stats.tx_packets++;
                } /* if-else */

                    /* Free the original skb. */
                dev_kfree_skb(pDequeueTxDesc->tx_skbuff, FREE_WRITE);
                pDequeueTxDesc->tx_skbuff =0;

                tpadapter->nNumUnusedTxDesc++;/*update unused buffer coount*/
                pDequeueTxDesc = pDequeueTxDesc->pNext;
            }/* while */
            tpadapter->pDequeueTxDesc = pDequeueTxDesc;

           if (tpadapter->tx_full && dev->tbusy) {
               /* The ring is no longer full, clear tbusy. */
               tpadapter->tx_full = 0;
               dev->tbusy = 0;
               mark_bh(NET_BH);
           }/* if */

        } /* if tx_done interrupt */

                /* Log errors. */
        if (csr5 & 0x8000) {    /* Abnormal error summary bit. */
            if (csr5 & 0x0008) tpadapter->stats.tx_errors++; /* Tx babble. */
                if (csr5 & 0x0020) { /* Tx FIFO underflow. */
                    tpadapter->csr6 |= 0x00200000;  /* Reconfigure to store-n-forward. */
                    /* Restart the transmit process. */
                    outl(tpadapter->csr6 | 0x0002, ioaddr + CSR6);
                    outl(tpadapter->csr6 | 0x2002, ioaddr + CSR6);
                }/* if */
            if (csr5 & 0x0100) {            /* Missed a Rx frame. */
                tpadapter->stats.rx_errors++;
                tpadapter->stats.rx_missed_errors += inl(ioaddr + CSR8) & 0xffff;
            } /* if */
            if (csr5 & 0x0800)
                printk("%s: Something Wicked happened! %8.8x.\n", dev->name, csr5);
                /* Clear all error sources, included undocumented ones! */
            outl(0x000f7ba, ioaddr + CSR5);
        }/* if */

        if (--boguscnt < 0) {
            printk("%s: Too much work at interrupt, csr5=0x%8.8x.\n", dev->name, csr5);
            /* Clear all interrupt sources. */
            outl(0x0001ffff, ioaddr + CSR5);
            break;
        }/* if */
    } while (1);

    if (tulip_debug > 3)
        printk("%s: exiting interrupt, csr5=%#4.4x.\n", dev->name, inl(ioaddr + CSR5));
    dev->interrupt = 0;
    return;
}

static int tulip_close(struct device *dev)
{
    int ioaddr = dev->base_addr;
    tADAPTER    *tpadapter = (tADAPTER *)dev->priv;
    tPTX_DESC   tx_desc;
    tPRX_DESC   rx_desc;
    int i;

    dev->start = 0;
    dev->tbusy = 1;

    DBG_CLOSE("%s: Shutting down ethercard, status csr5= %8.8x\n",
                       dev->name, inl(ioaddr + CSR5));

    /* Disable interrupts by clearing the interrupt mask. */
    outl(0x00000000, ioaddr + CSR7);
    /* Stop the chip's Tx and Rx processes. */
    outl(inl(ioaddr + CSR6) & ~0x2002, ioaddr + CSR6);

    tpadapter->stats.rx_missed_errors += inl(ioaddr + CSR8) & 0xffff;

    del_timer(&tpadapter->timer);

#ifdef SA_SHIRQ
    free_irq(dev->irq, dev);
#else
    free_irq(dev->irq);
    irq2dev_map[dev->irq] = 0;
#endif

    rx_desc = tpadapter->pRxRing;
    /* Free all the skbuffs in the Rx queue. */
    for (i = 0; i < RX_RING_SIZE; i++) {
        struct sk_buff *skb = rx_desc->rx_skbuff;
        rx_desc->rx_skbuff = 0;
        rx_desc->wStatus = 0;              /* Not owned by Tulip chip. */
        rx_desc->dwCommand =0;
        rx_desc->wFrameLength = 0;
        rx_desc->dwAddress1 = 0xBADF00D0; /* An invalid address. */
        if (skb) {
#if LINUX_VERSION_CODE < 0x20100
            skb->free = 1;
#endif
            dev_kfree_skb(skb, FREE_WRITE);
        } /* if */
        rx_desc= rx_desc->pNext;
    }/* for */

    tx_desc = tpadapter->pTxRing;
    for (i = 0; i < TX_RING_SIZE; i++) {
        if (tx_desc->tx_skbuff)
            dev_kfree_skb(tx_desc->tx_skbuff, FREE_WRITE);
        tx_desc->tx_skbuff = 0;
        tx_desc=tx_desc->pNext;
    }   /*for*/

    return 0;
}

static struct enet_statistics *tulip_get_stats(struct device *dev)
{
    tADAPTER    *tpadapter = (tADAPTER *)dev->priv;
    int ioaddr = dev->base_addr;

    if (dev->start)
        tpadapter->stats.rx_missed_errors += inl(ioaddr + CSR8) & 0xffff;

    return &tpadapter->stats;
}

/* Set or clear the multicast filter for this adaptor.
   Note that we only use exclusion around actually queueing the
   new frame, not around filling tp->setup_frame.  This is non-deterministic
   when re-entered but still correct. */

/* The little-endian AUTODIN32 ethernet CRC calculation.
   N.B. Do not use for bulk data, use a table-based routine instead.
   This is common code and should be moved to net/core/crc.c */
static unsigned const ethernet_polynomial_le = 0xedb88320U;
static inline unsigned ether_crc_le(int length, unsigned char *data)
{
    unsigned int crc = 0xffffffff;  /* Initial value. */
    while(--length >= 0) {
        unsigned char current_octet = *data++;
        int bit;
        for (bit = 8; --bit >= 0; current_octet >>= 1) {
            if ((crc ^ current_octet) & 1) {
                crc >>= 1;
                crc ^= ethernet_polynomial_le;
            } else
                crc >>= 1;
        }
    }
    return crc;
}


static void
#ifdef NEW_MULTICAST
set_multicast_list(struct device *dev)
#else
static void set_multicast_list(struct device *dev, int num_addrs, void *addrs);
#endif
{
    int ioaddr = dev->base_addr;
    int csr6 = inl(ioaddr + CSR6) & ~0x00D5;
    tADAPTER    *tpadapter = (tADAPTER *)dev->priv;
    DBG_MULTICAST("Setting multicast list....\n");

    tpadapter->csr6 &= ~0x00D5;
    if (dev->flags & IFF_PROMISC) {                 /* Set promiscuous. */
        outl(csr6 | 0x00C0, ioaddr + CSR6);
        /* Unconditionally log net taps. */
        printk("%s: Promiscuous mode enabled.\n", dev->name);
        tpadapter->csr6 |= 0xC0;
    } else if ((dev->mc_count > 1000)  ||  (dev->flags & IFF_ALLMULTI)) {
        /* Too many to filter perfectly -- accept all multicasts. */
        outl(csr6 | 0x0080, ioaddr + CSR6);
        tpadapter->csr6 |= 0x80;
    } else {
        u32 *setup_frm = (u32*)tpadapter->SetupBuffer;
        struct dev_mc_list *mclist;
        u16 *eaddrs;
        u32 tx_flags;
        int i;

        if (dev->mc_count > 14) { /* Must use a multicast hash table. */
            u16 hash_table[32];
            memset(hash_table, 0, sizeof(hash_table));
            for (i = 0, mclist = dev->mc_list; mclist && i < dev->mc_count;
                i++, mclist = mclist->next)
                set_bit(ether_crc_le(ETH_ALEN, mclist->dmi_addr) & 0x1ff, hash_table);
            /* Copy the hash table to the setup frame.
               NOTE that only the LOW SHORTWORD of setup_frame[] is valid!
               This code may require tweaking for non-x86 architectures!  */
            for (i = 0; i < 32; i++)
                *setup_frm++ = hash_table[i];
            setup_frm += 7;
            tx_flags = 0x08400000|192|0x01000000;
            /* Too clever: i > 15 for fall-though. */
        } else {
            /* We have <= 15 addresses so we can use the wonderful
               16 address perfect filtering of the Tulip. */
            for (i = 0, mclist = dev->mc_list; i < dev->mc_count;
                 i++, mclist = mclist->next) {
                /* Note that only the low shortword of setup_frame[] is valid!
                   This code may require tweaking for non-x86 architectures! */
                eaddrs = (u16 *)mclist->dmi_addr;
                *setup_frm++ = *eaddrs++;
                *setup_frm++ = *eaddrs++;
                *setup_frm++ = *eaddrs++;
            } /*for*/
            /* Fill the rest of the table with our physical address.
               Once again, only the low shortword or setup_frame[] is valid! */
            *setup_frm++ = 0xffff;
            *setup_frm++ = 0xffff;
            *setup_frm++ = 0xffff;
            tx_flags = 0x08000000|192|0x01000000;
        }/*else*/
        eaddrs = (u16 *)dev->dev_addr;
        do {
            *setup_frm++ = eaddrs[0];
            *setup_frm++ = eaddrs[1];
            *setup_frm++ = eaddrs[2];
        } while (++i < 15);
        
        /* Now add this frame to the Tx list. */
        if (tpadapter->nNumUnusedTxDesc > TX_RING_SIZE - 2) {
            /* Same setup recently queued, we need not add it. */
            } else {
                unsigned long tx_flags;
                tPTX_DESC   tx_desc = tpadapter->pEnqueueTxDesc;
                save_flags(tx_flags); cli();

                /* Put the setup frame on the Tx list. */
                if (tx_desc == (tx_desc + sizeof(tPTX_DESC)*(TX_RING_SIZE-1)))
                        tx_flags |= 0x02000000; /* Wrap ring. */

                tx_desc->tx_skbuff = 0;
                tx_desc->wOwnBit = 0x8000;
                tx_desc->wStatus = 0x0000;
                tx_desc->dwCommand = tx_flags;
                tx_desc->dwAddress1= virt_to_bus(tpadapter->SetupBuffer);

                if (tpadapter->nNumUnusedTxDesc<= TX_RING_SIZE - 2) {
                    dev->tbusy = 1;
                    tpadapter->tx_full = 1;
                }
                restore_flags(tx_flags);
                /* Trigger an immediate transmit demand. */
                outl(0, ioaddr + CSR1);
        } /*if-else*/
        outl(csr6 | 0x0000, ioaddr + CSR6);
    } /*if-else*/
}

#ifdef MODULE

/* An additional parameter that may be passed in... */
static int debug = -1;

int init_module(void)
{
    int cards_found;

    if (debug >= 0)
        tulip_debug = debug;

    root_dlkpci_dev = NULL;
    cards_found = tulip_probe(0);

    return cards_found ? 0 : -ENODEV;
}


void cleanup_module(void)
{
    struct device *next_dev;

    /* No need to check MOD_IN_USE, as sys_delete_module() checks. */
    while (root_dlkpci_dev) {
        next_dev = ((tADAPTER *)root_dlkpci_dev->priv)->next_module;
        unregister_netdev(root_dlkpci_dev);
        release_region(root_dlkpci_dev->base_addr, TULIP_TOTAL_SIZE);
        kfree(root_dlkpci_dev);
        root_dlkpci_dev = next_dev;
    }   /*while*/
}

#endif  /* MODULE */
/*
 * Local variables:
 *  compile-command: "gcc -DMODULE -D__KERNEL__ -I/usr/src/linux/net/inet -Wall -Wstrict-prototypes -O6 -c dlkpci.c"
 *  c-indent-level: 4
 *  tab-width: 4
 * End:
 */

