/*
 *  IOACCESS.H
 *
 *  Description:    This file define the I/O access macros for
 *                  the platforms we supports.
 *
 *  Copyright (c) 1997 by D-Link Corporation. All rights reserved.
 *
 *  Created by David Hsieh at 970306
 *
 *  Revision History:
 *
 *  Name        Date        Ver     Brief
 *  ----------- ----------- ------- ------------------------------------------
 *  David Hsieh 970306      v1.00   Create
 *  David Hsieh 970414      v1.00   Add for C ODI I/O access.
 *  David Hsieh 970507      v1.00   Use tPUB structure in ODI32.
 *  Becker Horng 970715     v1.01   Added Condition compilication for Linux IO
 *                                  operation.
 *
 */
#ifndef IOACCESS_H
#define IOACCESS_H

#ifndef OSDEF_H
#include "osdef.h"
#endif

#if OS_TYPE == NDIS
#define mInD(_Port,_Value)          NdisRawReadPortUlong(_Port,_Value)
#define mInW(_Port,_Value)          NdisRawReadPortUshort(_Port,_Value)
#define mInB(_Port,_Value)          NdisRawReadPortUchar(_Port,_Value)
#define mOutD(_Port,_Value)         NdisRawWritePortUlong(_Port,_Value)
#define mOutW(_Port,_Value)         NdisRawWritePortUshort(_Port,_Value)
#define mOutB(_Port,_Value)         NdisRawWritePortUchar(_Port,_Value)
#endif

#if OS_TYPE == DOS
#define mInD(_Port,_Value)          {*(_Value)=inpd(_Port);}
#define mInW(_Port,_Value)          {*(_Value)=inpw(_Port);}
#define mInB(_Port,_Value)          {*(_Value)=inp (_Port);}
#define mOutD(_Port,_Value)         outpd(_Port,_Value)
#define mOutW(_Port,_Value)         outpw(_Port,_Value)
#define mOutB(_Port,_Value)         outp (_Port,_Value)
#endif

#if OS_TYPE == SCO_UNIX | OS_TYPE == ATT_UNIX
#define mInD(_Port,_Value)          {*(_Value)=ind(_Port);}
#define mInW(_Port,_Value)          {*(_Value)=inw(_Port);}
#define mInB(_Port,_Value)          {*(_Value)=inb(_Port);}
#define mOutD(_Port,_Value)         outd(_Port,_Value)
#define mOutW(_Port,_Value)         outw(_Port,_Value)
#define mOutB(_Port,_Value)         outb(_Port,_Value)
#endif

#if OS_TYPE == ODI32
#define mInD(_Port,_Value)          {*(_Value)=In32(pPub->pBusTag,(tPVOID)_Port);}
#define mInW(_Port,_Value)          {*(_Value)=In16(pPub->pBusTag,(tPVOID)_Port);}
#define mInB(_Port,_Value)          {*(_Value)=In8(pPub->pBusTag,(tPVOID)_Port);}
#define mOutD(_Port,_Value)         Out32(pPub->pBusTag,(tPVOID)_Port,_Value)
#define mOutW(_Port,_Value)         Out16(pPub->pBusTag,(tPVOID)_Port,_Value)
#define mOutB(_Port,_Value)         Out8(pPub->pBusTag,(tPVOID)_Port,_Value)
#endif

/* added by Becker to support Linux operating system */
#if OS_TYPE == LINUX

#define mInD(_Port,_Value)          {*(_Value)=inl(_Port);}
#define mInW(_Port,_Value)          {*(_Value)=inw(_Port);}
#define mInB(_Port,_Value)          {*(_Value)=inb(_Port);}
#define mOutD(_Port,_Value)         outl(_Value,_Port)
#define mOutW(_Port,_Value)         outw(_Value,_Port)
#define mOutB(_Port,_Value)         outb(_Value,_Port)

#endif  /* LINUX */


#endif
