/*
 *  MII.H
 *
 *  Description:    This file defines all the constants and structures used
 *                  for MII PHY access.
 *
 *  Copyright (c) 1997 by D-Link Corporation. All right reserved.
 *
 *  Created by XXXX
 *
 *  Revision History:
 *
 *  Name        Date        Ver     Brief
 *  ----------- ----------- ------- ------------------------------------------
 *  Ken Vong    97-04-10    1.00
 *
 */
#ifndef MII_H
#define MII_H


/**************************
 *
 *  Mii Phy Medium
 *
 **************************/
#define MAX_PHY_REGS                32
#define MAX_PHY_ADDR                32


/* PHY ID definition */
#define PHYID_DP83840       0x20005C00UL        /* National DP83840 */
#define PHYID_DP83840A      0x20005C01UL        /* National DP83840A */
#define PHYID_LXT970        0x78100000UL        /* Level One */
#define GENERIC_PHY         0xFFFFFFFFUL



/**************************
 *
 *  Variable
 *
 **************************/
#define PHY_ADDR_ALIGN              23
#define REG_ADDR_ALIGN              18

#define MII_DELAY                   1   /* 1us */
#define MII_MDO_BITS_SHIFT          17
#define MII_MDI_BITS_SHIFT          19

#define MII_WRITE                   (tULONG)0x00002000
#define MII_WRITE_THREE_STATE       (tULONG)0x00042000
#define MII_READ                    (tULONG)0x00044000
#define MII_DATA_0                  (tULONG)0x00000000
#define MII_DATA_1                  (tULONG)0x00020000
#define MII_CLK                     (tULONG)0x00010000

#define MII_PRE                     (tULONG)0xFFFFFFFF
#define MII_READ_FRAME              (tULONG)0x60000000
#define MII_WRITE_FRAME             (tULONG)0x50020000

#define MII_RESET_TIME_OUT          10000

/* MII Phy Registers's */
#define MII_BMCR                    0
#define MII_BMSR                    1
#define MII_PHY_ID1                 2
#define MII_PHY_ID2                 3
#define MII_ANAR                    4
#define MII_ANLPAR                  5
#define MII_ANER                    6

typedef enum tMII_STATUS
{
   eADMIN_RESET,
   eADMIN_OPERATIONAL,
   eADMIN_LOOP_BACK,
   eADMIN_POWER_DOWN,
   eADMIN_ISOLATE
}tMII_STATUS,*tPMII_STATUS;

/* BMCR */
#define MII_BMCR_RESET              ((tUSHORT) 0x8000)
#define MII_BMCR_LOOP_BACK          ((tUSHORT) 0x4000)
#define MII_BMCR_SPEED100           ((tUSHORT) 0x2000)
#define MII_BMCR_NWAY_ENABLE        ((tUSHORT) 0x1000)
#define MII_BMCR_POWER_DOWN         ((tUSHORT) 0x0800)
#define MII_BMCR_ISOLATE            ((tUSHORT) 0x0400)
#define MII_BMCR_RESTART_NWAY       ((tUSHORT) 0x0200)
#define MII_BMCR_DUPLEX_MODE        ((tUSHORT) 0x0100)
#define MII_BMCR_COL_TEST           ((tUSHORT) 0x0080)
#define MII_BMCR_RESERVED           ((tUSHORT) 0x007F)

#define MII_BMCR_SPEED10            ((tUSHORT) 0x0000)

/* BMSR */
#define MII_BMSR_100BASETX_FD       ((tUSHORT) 0x4000)
#define MII_BMSR_100BASETX          ((tUSHORT) 0x2000)
#define MII_BMSR_10BASET_FD         ((tUSHORT) 0x1000)
#define MII_BMSR_10BASET            ((tUSHORT) 0x0800)
#define MII_BMSR_NWAY_COMPLETE      ((tUSHORT) 0x0020)
#define MII_BMSR_NWAY_ABILITY       ((tUSHORT) 0x0008)
#define MII_BMSR_LINK               ((tUSHORT) 0x0004)
#define MII_BMSR_EXT_CAP            ((tUSHORT) 0x0001)

/* ANAR & ANLPAR */
#define MII_ANAR_100BASETX_FD       ((tUSHORT) 0x0100)
#define MII_ANAR_100BASETX          ((tUSHORT) 0x0080)
#define MII_ANAR_10BASET_FD         ((tUSHORT) 0x0040)
#define MII_ANAR_10BASET            ((tUSHORT) 0x0020)

#define MII_ANAR_SELECTOR           ((tUSHORT) 0x0001)

/* MII_MEDIA_TYPE cConnection settings */
enum
{
    MII_100BASE_TX_FD = 1,
    MII_100BASE_TX,
    MII_10BASE_T_FD,
    MII_10BASE_T
};


/* MII_MEDIA_TYPE cControl settings */
#define MII_NWAY_ENABLE         0x1
#define MII_NWAY_AUTOSENSE      0x2

/* Media type structure in MiiPhy */

typedef struct tMII_MEDIA_TYPE
{
    tBYTE                   cConnection;
    tBYTE                   cControl;
} tMII_MEDIA_TYPE, *tPMII_MEDIA_TYPE;

/* MiiPhy information block structure */
typedef struct tMII_PHY_INFO
{
    /* Phy information */
    tBOOLEAN                    bStructValid;
    tUSHORT                     wMIIPhyAddress;
    tULONG                      dwPhyID;
    tUSHORT                     wLocalAdvertisement;
    tUSHORT                     wPhyRegs[MAX_PHY_REGS];

} tMII_PHY_INFO, *tPMII_PHY_INFO;
#endif
