/*
 *  SROM.H
 *
 *  Description:    This file defines the constants and structure used in
 *                  SROM module.
 *
 *  Copyright (c) 1997 by D-Link Corporation. All rights reserved.
 *
 *  Created by David Hsieh.
 *
 *  Revision History:
 *
 *  Name        Date        Ver         Brief
 *  ----------- ----------- ----------- --------------------------------------
 *
 *
 */
#ifndef SROM_H
#define SROM_H


/*
 *  Instruction set for NM93C46
 *
 *  Instruction SB Op code Address Data                 Comment
 *  ----------- -- ------- ------- -------------------- -----------------------
 *  READ         1    10   A5 - A0                      Read Register [A5-A0]
 *  WRITE        1    01   A5 - A0 D15      -       D0  Write Register [A5-A0]
 *  ERASE        1    11   A5 - A0                      Erase Register [A5-A0]
 *  EWEN         1    00   11XXXX                       Erase/Write Enable
 *  EWDS         1    00   00XXXX                       Erase/Write Disable
 *  ERAL         1    00   10XXXX                       Erase All Registers
 *  WRAL         1    00   01XXXX  D15      -       D0  Write All Registers
 *
 *
 *  Instruction set for NM93C56
 *
 *  Instruction SB Op code Address  Data                Comment
 *  ----------- -- ------- -------- ------------------- -----------------------
 *  READ         1    10   A7 - A0                      Read Register [A7-A0]
 *  WRITE        1    01   A7 - A0  D15      -      D0  Write Register [A7-A0]
 *  ERASE        1    11   A7 - A0                      Erase Register [A7-A0]
 *  EWEN         1    00   11XXXXXX                     Erase/Write Enable
 *  EWDS         1    00   00XXXXXX                     Erase/Write Disable
 *  ERAL         1    00   10XXXXXX                     Erase All Registers
 *  WRAL         1    00   01XXXXXX D15      -      D0  Write All Registers
 *
 */
#define SROM46_MAX_CYCLES               25      /* 1 + 2 + 6 + 16 */
#define SROM46_OP_ADDRCNT               9       /* 1 + 2 + 6 */
#define SROM46_OP_READ                  0x180   /* 1 1000 0000 */
#define SROM46_OP_WRITE                 0x140   /* 1 0100 0000 */
#define SROM46_OP_ERASE                 0x1C0   /* 1 1100 0000 */
#define SROM46_OP_EWEN                  0x130   /* 1 0011 0000 */
#define SROM46_OP_EWDS                  0x100   /* 1 0000 0000 */

#define SROM56_MAX_CYCLES               27      /* 1 + 2 + 8 + 16 */
#define SROM56_OP_ADDRCNT               11      /* 1 + 2 + 8 */
#define SROM56_OP_READ                  0x600   /* 0110 0000 0000 */
#define SROM56_OP_WRITE                 0x500   /* 0101 0000 0000 */
#define SROM56_OP_ERASE                 0x700   /* 0111 0000 0000 */
#define SROM56_OP_EWEN                  0x4C0   /* 0100 1100 0000 */
#define SROM56_OP_EWDS                  0x400   /* 0100 0000 0000 */


/* Read/Write Serail EEPROM port define */
#define RWSROM_PORT_INDEX               9       /* CSR 9 */
#define SROM_DELAY                      5       /* micro second (us) */

#define CSR_READ                        0x4000
#define CSR_WRITE                       0x2000
#define SEL_SROM                        0x0800
#define DATA_1                          0x0004
#define DATA_0                          0x0000
#define CLK                             0x0002
#define CS                              0x0001
#define SROM_DO_BIT_MASK                0x0008

#endif
